"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const path = require("path");
const Logger_1 = require("./Logger");
// import { Log } from "./Logger";
/**
 * Append toAppend to start, removing the last segment of start if the first segment of toAppend matches it.
 *
 * appendPathWithoutDupe("/home/tim/microclimate-workspace/", "/microclimate-workspace/myproject")
 *      -> "/home/tim/microclimate-workspace/myproject"
 */
function appendPathWithoutDupe(start, toAppend) {
    // Remove end of start / if present
    if (start.endsWith(path.sep)) {
        start = start.substring(0, start.length);
    }
    // Remove start of toAppend / if present
    if (toAppend.startsWith(path.sep)) {
        toAppend = toAppend.substring(1, toAppend.length + 1);
    }
    const lastStartSegment = lastPathSegment(start);
    if (toAppend.startsWith(lastStartSegment)) {
        start = start.substring(0, start.length - lastStartSegment.length);
    }
    return path.join(start, toAppend);
}
exports.appendPathWithoutDupe = appendPathWithoutDupe;
/**
 * Returns the last segment of the given path, with no starting slash.
 * Trailing slash is kept if present.
 *
 * lastPathSegment("/home/tim/test/dir/") -> "dir/"
 */
function lastPathSegment(p) {
    return p.substr(p.lastIndexOf(path.sep) + 1);
}
exports.lastPathSegment = lastPathSegment;
function uppercaseFirstChar(input) {
    return input.charAt(0).toUpperCase() + input.slice(1);
}
exports.uppercaseFirstChar = uppercaseFirstChar;
/**
 * Returns a wrapper promise which runs the given promise with the given timeout.
 * If the timeout expires before the given promise is fulfilled, the wrapper promise rejects with the given message.
 *
 * If the promise resolves or rejects before the timeout,
 * the wrapper promise resolves or rejects with the same result as the inner promise.
 */
function promiseWithTimeout(promise, timeoutMS, rejectMsg) {
    return new Promise((resolve, reject) => {
        setTimeout(() => reject(rejectMsg), timeoutMS);
        promise
            .then((result) => resolve(result))
            .catch((err) => reject(err));
    });
}
exports.promiseWithTimeout = promiseWithTimeout;
function isGoodDate(date) {
    return !isNaN(date.valueOf());
}
exports.isGoodDate = isGoodDate;
function isGoodStatusCode(statusCode) {
    return statusCode != null && !isNaN(statusCode) && statusCode >= 200 && statusCode < 400;
}
exports.isGoodStatusCode = isGoodStatusCode;
function isGoodPort(port) {
    return port != null && !isNaN(port) && Number.isInteger(port) && port > 1024 && port < 65536;
}
exports.isGoodPort = isGoodPort;
/**
 * Convert a ConnectionInfo to an HTTP URI.
 */
function buildMCUrl(connInfo) {
    return vscode_1.Uri.parse(`http://${connInfo.host}:${connInfo.port}`); // non-nls
}
exports.buildMCUrl = buildMCUrl;
/**
 * Convert a URI to a ConnectionInfo (for saving to Settings).
 * A URI type with a 'port' field would be preferable, but vscode does not have this.
 */
function getConnInfoFrom(url) {
    const colonIndex = url.authority.indexOf(":"); // non-nls
    const host = url.authority.substring(0, colonIndex);
    const portStr = url.authority.substring(colonIndex + 1, url.authority.length);
    const port = Number(portStr);
    if (!isGoodPort(port)) {
        Logger_1.default.e(`Bad port ${portStr} passed to getConnInfoFrom`);
    }
    // Log.i(`Loaded connection info host ${host} port ${port}`);
    const result = {
        host: host,
        port: port
    };
    return result;
}
exports.getConnInfoFrom = getConnInfoFrom;
//# sourceMappingURL=MCUtil.js.map