"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCUtil = require("../MCUtil");
const CommandUtil_1 = require("../command/CommandUtil");
const Resources_1 = require("../constants/Resources");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const Logger_1 = require("../Logger");
const ProjectType_1 = require("../microclimate/project/ProjectType");
const DebugUtils_1 = require("../microclimate/project/DebugUtils");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const STRING_NS = StringNamespaces_1.default.DEBUG;
/**
 * Attach the debugger to the given project. Returns if we attached the debugger successfully.
 *
 * NOTE: Do not throw or reject from this function in case this command is invoked directly - vscode won't handle the rejection.
 * Show the user our error message here instead, and return a success status since we're also calling this function from the restart project code.
 */
function attachDebuggerCmd(project, isRestart = false) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("attachDebuggerCmd");
        if (project == null) {
            const selected = yield CommandUtil_1.promptForProject(...ProjectState_1.default.getDebuggableStates());
            if (selected == null) {
                // user cancelled
                Logger_1.default.d("User cancelled project prompt");
                return false;
            }
            project = selected;
        }
        try {
            if (isRestart) {
                Logger_1.default.d("Attach debugger runnning as part of a restart");
                // Intermittently for restarting Microprofile projects, the debugger will try to connect too soon,
                // so add an extra delay if it's MP and Starting.
                // This doesn't really slow anything down because the server is still starting anyway.
                const libertyDelayMs = 2500;
                if (project.type.type === ProjectType_1.default.Types.MICROPROFILE && project.state.appState === ProjectState_1.default.AppStates.DEBUG_STARTING) {
                    Logger_1.default.d(`Waiting extra ${libertyDelayMs}ms for Starting Liberty project`);
                    const delayPromise = new Promise((resolve) => setTimeout(resolve, libertyDelayMs));
                    const preDebugDelayMsg = translator_1.default.t(STRING_NS, "waitingBeforeDebugAttachStatusMsg", { projectName: project.name });
                    vscode.window.setStatusBarMessage(`${Resources_1.default.getOcticon(Resources_1.default.Octicons.bug, true)} ${preDebugDelayMsg}`, delayPromise);
                    yield delayPromise;
                }
            }
            // This should be longer than the timeout we pass to VSCode through the debug config, or the default (whichever is longer).
            const debugConnectTimeoutS = 60;
            Logger_1.default.d(`${project.name} appears to be ready for debugging`);
            const startDebugWithTimeout = MCUtil.promiseWithTimeout(DebugUtils_1.default.startDebugSession(project), debugConnectTimeoutS * 1000, translator_1.default.t(STRING_NS, "didNotConnectInTime", { timeoutS: debugConnectTimeoutS }));
            const connectingMsg = translator_1.default.t(STRING_NS, "connectingToProject", { projectName: project.name });
            vscode.window.setStatusBarMessage(`${Resources_1.default.getOcticon(Resources_1.default.Octicons.bug, true)} ${connectingMsg}`, // non-nls
            startDebugWithTimeout);
            // will throw error if connection fails or timeout
            const successMsg = yield startDebugWithTimeout;
            Logger_1.default.i("Debugger attach success:", successMsg);
            vscode.window.showInformationMessage(successMsg);
            return true;
        }
        catch (err) {
            const debugUrl = project.debugUrl;
            let failMsg;
            if (debugUrl != null) {
                failMsg = translator_1.default.t(STRING_NS, "failedToAttachWithUrl", { projectName: project.name, debugUrl });
            }
            else {
                failMsg = translator_1.default.t(STRING_NS, "failedToAttach", { projectName: project.name });
            }
            const extraErrMsg = err.message ? err.message : ""; // non-nls
            Logger_1.default.e(failMsg, extraErrMsg);
            if (extraErrMsg != null && extraErrMsg.length > 0) {
                failMsg += translator_1.default.t(STRING_NS, "errDetailSeparator") + extraErrMsg;
            }
            vscode.window.showErrorMessage(failMsg);
            return false;
        }
    });
}
exports.default = attachDebuggerCmd;
//# sourceMappingURL=AttachDebuggerCmd.js.map