"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Logger_1 = require("../Logger");
const Project_1 = require("../microclimate/project/Project");
const Connection_1 = require("../microclimate/connection/Connection");
const ConnectionManager_1 = require("../microclimate/connection/ConnectionManager");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const Commands_1 = require("../constants/Commands");
const NewConnectionCmd = require("./NewConnectionCmd");
const OpenWorkspaceFolderCmd_1 = require("./OpenWorkspaceFolderCmd");
const RestartProjectCmd_1 = require("./RestartProjectCmd");
const OpenInBrowserCmd_1 = require("./OpenInBrowserCmd");
const RequestBuildCmd_1 = require("./RequestBuildCmd");
const ToggleEnablementCmd_1 = require("./ToggleEnablementCmd");
const RemoveConnectionCmd_1 = require("./RemoveConnectionCmd");
const ContainerShellCmd_1 = require("./ContainerShellCmd");
const ProjectOverviewCmd_1 = require("./ProjectOverviewCmd");
const AttachDebuggerCmd_1 = require("./AttachDebuggerCmd");
const OpenLogCmd_1 = require("./OpenLogCmd");
const ToggleAutoBuildCmd_1 = require("./ToggleAutoBuildCmd");
const OpenAppMonitor_1 = require("./OpenAppMonitor");
const RefreshConnectionCmd_1 = require("./RefreshConnectionCmd");
const NewMCProjectCmd_1 = require("./NewMCProjectCmd");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const ValidateCmd_1 = require("./ValidateCmd");
const HideLogsCmd_1 = require("./HideLogsCmd");
function createCommands() {
    // Register our commands here
    // The first parameter must match the command ID as declared in package.json
    // the second parameter is the callback function, which is passed the user's selection, which is either:
    // - undefined (if run from command palette)
    // - or the user's selected TreeView object (if run from the context menu) -> IE either a Project or Connection
    return [
        vscode.commands.registerCommand(Commands_1.default.NEW_CONNECTION, () => NewConnectionCmd.newConnectionCmd()),
        vscode.commands.registerCommand(Commands_1.default.NEW_DEFAULT_CONNECTION, () => NewConnectionCmd.newConnectionCmd(NewConnectionCmd.DEFAULT_CONNINFO)),
        vscode.commands.registerCommand(Commands_1.default.REMOVE_CONNECTION, (selection) => RemoveConnectionCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.REFRESH_CONNECTION, (selection) => RefreshConnectionCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.CREATE_MC_PROJECT, (selection) => NewMCProjectCmd_1.default(selection, true)),
        vscode.commands.registerCommand(Commands_1.default.IMPORT_MC_PROJECT, (selection) => NewMCProjectCmd_1.default(selection, false)),
        vscode.commands.registerCommand(Commands_1.default.OPEN_WS_FOLDER, (selection) => OpenWorkspaceFolderCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.ATTACH_DEBUGGER, (selection) => AttachDebuggerCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.RESTART_RUN, (selection) => RestartProjectCmd_1.default(selection, false)),
        vscode.commands.registerCommand(Commands_1.default.RESTART_DEBUG, (selection) => RestartProjectCmd_1.default(selection, true)),
        vscode.commands.registerCommand(Commands_1.default.OPEN_IN_BROWSER, (selection) => OpenInBrowserCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.REQUEST_BUILD, (selection) => RequestBuildCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.TOGGLE_AUTOBUILD, (selection) => ToggleAutoBuildCmd_1.default(selection)),
        // Enable and disable AB are the same as Toggle AB - they are just presented to the user differently.
        vscode.commands.registerCommand(Commands_1.default.ENABLE_AUTOBUILD, (selection) => ToggleAutoBuildCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.DISABLE_AUTOBUILD, (selection) => ToggleAutoBuildCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.OPEN_APP_LOG, (selection) => OpenLogCmd_1.default(selection, true)),
        vscode.commands.registerCommand(Commands_1.default.OPEN_BUILD_LOG, (selection) => OpenLogCmd_1.default(selection, false)),
        vscode.commands.registerCommand(Commands_1.default.HIDE_LOGS, () => HideLogsCmd_1.default()),
        vscode.commands.registerCommand(Commands_1.default.DISABLE_PROJECT, (selection) => ToggleEnablementCmd_1.default(selection, false)),
        vscode.commands.registerCommand(Commands_1.default.ENABLE_PROJECT, (selection) => ToggleEnablementCmd_1.default(selection, true)),
        vscode.commands.registerCommand(Commands_1.default.CONTAINER_SHELL, (selection) => ContainerShellCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.PROJECT_OVERVIEW, (selection) => ProjectOverviewCmd_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.OPEN_APP_MONITOR, (selection) => OpenAppMonitor_1.default(selection)),
        vscode.commands.registerCommand(Commands_1.default.VALIDATE, (selection) => ValidateCmd_1.default(selection))
    ];
}
exports.createCommands = createCommands;
// Some commands require a project or connection to be selected,
// if they're launched from the command palette we have to ask which resource they want to run the command on.
// The functions below handle this use case.
/**
 *
 * @param acceptableStates - If at least one state is passed, only projects in one of these states will be presented to the user.
 */
function promptForProject(...acceptableStates) {
    return __awaiter(this, void 0, void 0, function* () {
        const project = yield promptForResourceInner(false, true, false, ...acceptableStates);
        if (project instanceof Project_1.default) {
            return project;
        }
        else if (project != null) {
            // should never happen
            Logger_1.default.e("promptForProject received something other than a project back:", project);
        }
        // user cancelled, or error above
        return undefined;
    });
}
exports.promptForProject = promptForProject;
function promptForConnection(activeOnly) {
    return __awaiter(this, void 0, void 0, function* () {
        if (ConnectionManager_1.default.instance.connections.length === 1) {
            const onlyConnection = ConnectionManager_1.default.instance.connections[0];
            if (onlyConnection.isConnected || !activeOnly) {
                return onlyConnection;
            }
            // else continue to promptForResource, which will report if there are no suitable connections.
        }
        const connection = yield promptForResourceInner(true, false, activeOnly);
        if (connection instanceof Connection_1.default) {
            return connection;
        }
        else if (connection != null) {
            // should never happen
            Logger_1.default.e("promptForConnection received something other than a connection back:", connection);
        }
        // user cancelled, or error above
        return undefined;
    });
}
exports.promptForConnection = promptForConnection;
function promptForResource(activeConnectionsOnly, ...acceptableStates) {
    return __awaiter(this, void 0, void 0, function* () {
        return promptForResourceInner(true, true, activeConnectionsOnly, ...acceptableStates);
    });
}
exports.promptForResource = promptForResource;
/**
 * If !includeConnections, activeConnectionsOnly is ignored.
 * If !includeProjects, acceptableStates is ignored.
 */
function promptForResourceInner(includeConnections, includeProjects, activeConnectionsOnly, ...acceptableStates) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!includeConnections && !includeProjects) {
            // One of these must always be set
            Logger_1.default.e("Neither connection or projects are to be included!");
            return undefined;
        }
        else if (!includeProjects && acceptableStates.length > 0) {
            // This doesn't actually matter, but we're going to log this misuse anyway
            Logger_1.default.e("Not including projects, but acceptable states were specified!");
            acceptableStates = [];
        }
        else if (!includeConnections && activeConnectionsOnly) {
            // This doesn't actually matter, but we're going to log this misuse anyway
            Logger_1.default.e("Not including connections, but activeConnectionsOnly is set!");
        }
        const choices = [];
        const connections = ConnectionManager_1.default.instance.connections;
        if (includeConnections) {
            if (activeConnectionsOnly) {
                choices.push(...(connections.filter((conn) => conn.isConnected)));
            }
            else {
                choices.push(...connections);
            }
        }
        if (includeProjects) {
            // for now, assume if they want Started, they also accept Debugging. This may change.
            if (acceptableStates.includes(ProjectState_1.default.AppStates.STARTED) && !acceptableStates.includes(ProjectState_1.default.AppStates.DEBUGGING)) {
                acceptableStates.push(ProjectState_1.default.AppStates.DEBUGGING);
            }
            // same for Starting / Starting - Debug
            if (acceptableStates.includes(ProjectState_1.default.AppStates.STARTING) && !acceptableStates.includes(ProjectState_1.default.AppStates.DEBUG_STARTING)) {
                acceptableStates.push(ProjectState_1.default.AppStates.DEBUG_STARTING);
            }
            // Logger.log("Accept states", acceptableStates);
            // For each connection, get its project list, and filter by projects we're interested in.
            // then add the remaining projects to our QuickPick choices.
            for (const conn of connections) {
                let projects = yield conn.getProjects();
                if (acceptableStates.length > 0) {
                    // Filter out projects that are not in one of the acceptable states
                    projects = projects.filter((p) => acceptableStates.includes(p.state.appState));
                }
                choices.push(...projects);
            }
        }
        // If no choices are available, show a popup message
        if (choices.length === 0) {
            showNoValidResourcesMsg(includeProjects, includeConnections, acceptableStates);
            return undefined;
        }
        const selection = yield vscode.window.showQuickPick(choices, { canPickMany: false, });
        if (selection == null) {
            // user cancelled
            return undefined;
        }
        else if (selection instanceof Project_1.default) {
            return selection;
        }
        else if (selection instanceof Connection_1.default) {
            return selection;
        }
        else {
            Logger_1.default.e(`Unsupported type in promptForResource ${typeof (selection)}`);
            return undefined;
        }
    });
}
const STRING_NS = StringNamespaces_1.default.CMD_RES_PROMPT;
/**
 * Show a message stating that the command to be run can't be run on the current state of the workspace.
 * The message will be something like:
 * "There is no Microclimate connection, or Starting - Debug or Debugging project, to run this command on."
 */
function showNoValidResourcesMsg(includeProjects, includeConnections, acceptableStates) {
    let requiredStatesStr = ""; // non-nls
    const statesSpecified = acceptableStates.length !== 0;
    if (statesSpecified) {
        // this builds something like "Starting - Debug or Debugging", to represent the project states this command can run on.
        const sep = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "statesSeparator");
        requiredStatesStr += acceptableStates.map((state) => state.toString()).join(sep);
    }
    // In the case that the user runs a command but there is nothing valid to run that command on, we have to show a message.
    // There are several slightly different messages depending on the resource types the command accepts.
    let noValidResourcesMsg;
    if (includeProjects) {
        if (includeConnections) {
            if (statesSpecified) {
                noValidResourcesMsg = translator_1.default.t(STRING_NS, "noConnOrProjToRunOnWithStates", { states: requiredStatesStr });
            }
            else {
                noValidResourcesMsg = translator_1.default.t(STRING_NS, "noConnOrProjToRunOn");
            }
        }
        else if (statesSpecified) {
            noValidResourcesMsg = translator_1.default.t(STRING_NS, "noProjToRunOnWithStates", { states: requiredStatesStr });
        }
        else {
            noValidResourcesMsg = translator_1.default.t(STRING_NS, "noProjToRunOn");
        }
    }
    else if (includeConnections) {
        noValidResourcesMsg = translator_1.default.t(STRING_NS, "noConnToRunOn");
    }
    else {
        // this will never happen, it's checked for at the top of promptForResourceInner
        Logger_1.default.e("Neither connection or projects are to be included!");
        return;
    }
    vscode.window.showWarningMessage(noValidResourcesMsg);
}
//# sourceMappingURL=CommandUtil.js.map