"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCUtil = require("../MCUtil");
const ConnectionManager_1 = require("../microclimate/connection/ConnectionManager");
const Logger_1 = require("../Logger");
const Commands_1 = require("../constants/Commands");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const MCEnvironment_1 = require("../microclimate/connection/MCEnvironment");
exports.DEFAULT_CONNINFO = {
    host: "localhost",
    port: 9090
};
const STRING_NS = StringNamespaces_1.default.CMD_NEW_CONNECTION;
function newConnectionCmd(connInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("New connection command invoked");
        if (connInfo == null) {
            // Get it from the user
            connInfo = yield getConnectInfo();
            if (connInfo == null) {
                Logger_1.default.d("User cancelled entering connect info");
                // If the user didn't enter anything valid, exit here
                return undefined;
            }
        }
        const connection = yield tryAddConnection(connInfo);
        if (connection == null) {
            // user gave up
            return undefined;
        }
        yield offerToOpenWorkspace(connection);
        return connection;
    });
}
exports.newConnectionCmd = newConnectionCmd;
function getConnectInfo() {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("Prompting for connect info");
        // Only localhost is permitted. Uncomment this to (start to) support other hosts
        /*
        const inputOpts: vscode.InputBoxOptions = {
            prompt: Translator.t(STRING_NS, "enterMicroclimateHost"),
            value: DEFAULT_CONNINFO.host,
        };
        const hostname: string | undefined = await vscode.window.showInputBox(inputOpts);
    
        if (hostname == null) {
            // user cancelled
            return;
        }*/
        const host = exports.DEFAULT_CONNINFO.host;
        let tryAgain = true;
        let port;
        while (tryAgain) {
            const portStr = yield vscode.window.showInputBox({
                prompt: translator_1.default.t(STRING_NS, "enterMicroclimatePort"),
                value: exports.DEFAULT_CONNINFO.port.toString()
            });
            if (portStr == null) {
                // user cancelled
                return;
            }
            port = Number(portStr);
            if (!MCUtil.isGoodPort(port)) {
                const tryAgainBtn = translator_1.default.t(STRING_NS, "enterDifferentPortBtn");
                const result = yield vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "invalidPortNumber", { port: portStr }), tryAgainBtn);
                tryAgain = result === tryAgainBtn;
            }
            else {
                // they entered a good port, we can proceed.
                break;
            }
        }
        if (host == null || port == null) {
            // user never entered anything valid
            return undefined;
        }
        return { host, port };
    });
}
function offerToOpenWorkspace(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d(`offerToOpenWorkspace ${connection.mcUri} workspace=${connection.workspacePath}`);
        let inMcWorkspace = false;
        // See if the user has this connection's workspace open
        const wsFolders = vscode.workspace.workspaceFolders;
        if (wsFolders != null) {
            inMcWorkspace = wsFolders.some((folder) => folder.uri.fsPath.includes(connection.workspacePath.fsPath));
        }
        const successMsg = translator_1.default.t(STRING_NS, "connectionSucceeded", { connectionUri: connection.mcUri, workspacePath: connection.workspacePath.fsPath });
        Logger_1.default.i(successMsg);
        if (!inMcWorkspace) {
            const openWsBtn = translator_1.default.t(STRING_NS, "openWorkspaceBtn");
            // Provide a button to change their workspace to the microclimate-workspace if they wish
            vscode.window.showInformationMessage(successMsg, openWsBtn)
                .then((response) => {
                if (response === openWsBtn) {
                    vscode.commands.executeCommand(Commands_1.default.VSC_OPEN_FOLDER, connection.workspacePath);
                }
            });
        }
        else {
            // The user already has the workspace open, we don't have to do it for them.
            vscode.window.showInformationMessage(successMsg);
        }
    });
}
/**
 * Test connecting to the given host:port.
 *
 * If it fails, display a message to the user and allow them to either try to connect again with the same info,
 * or start the 'wizard' from the beginning to enter a new host/port.
 *
 * @returns The new Connection if we formed one, or undefined if we didn't.
 */
function tryAddConnection(connInfo, silent = false) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.i("TryAddConnection", connInfo);
        try {
            const uri = MCUtil.buildMCUrl(connInfo);
            const microclimateData = yield MCEnvironment_1.default.getEnvData(uri);
            // Connected successfully, now validate it's a good instance
            return yield onSuccessfulConnection(uri, connInfo.host, microclimateData);
        }
        catch (err) {
            const errMsg = err.message || err.toString();
            Logger_1.default.w("Connection test failed: " + errMsg);
            if (silent) {
                return undefined;
            }
            const editBtn = translator_1.default.t(STRING_NS, "editConnectionBtn");
            const retryBtn = translator_1.default.t(STRING_NS, "retryConnectionBtn");
            const response = yield vscode.window.showErrorMessage(errMsg, editBtn, retryBtn);
            if (response === editBtn) {
                // start again from the beginning (and this command instance will terminate after this function call exits)
                newConnectionCmd();
                return undefined;
            }
            else if (response === retryBtn) {
                // try to connect with the same host:port
                return yield tryAddConnection(connInfo);
            }
            return undefined;
        }
    });
}
exports.tryAddConnection = tryAddConnection;
/**
 * We've determined by this point that Microclimate is running at the given URI,
 * but we have to validate now that it's a new enough version.
 */
function onSuccessfulConnection(mcUri, host, mcEnvData) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.i("Microclimate ENV data:", mcEnvData);
        const rawVersion = mcEnvData.microclimate_version;
        const rawWorkspace = mcEnvData.workspace_location;
        const rawPlatform = mcEnvData.os_platform;
        // user_string and socket_namespace are the same on ICP, except latter starts with /
        // on local, user_string is null, and socket_namespace is "/default".
        // const rawUser: string = mcEnvData.user_string || "";
        const rawSocketNS = mcEnvData.socket_namespace || "";
        Logger_1.default.d("rawVersion from Microclimate is", rawVersion);
        Logger_1.default.d("rawWorkspace from Microclimate is", rawWorkspace);
        Logger_1.default.d("rawPlatform from Microclimate is", rawPlatform);
        Logger_1.default.d("rawSocketNS from Microclimate is", rawSocketNS);
        if (rawVersion == null || rawWorkspace == null) {
            Logger_1.default.e("Microclimate environment did not provide either version or workspace. Data provided is:", mcEnvData);
            throw new Error(translator_1.default.t(STRING_NS, "versionNotProvided", { requiredVersion: MCEnvironment_1.default.REQUIRED_VERSION_STR }));
        }
        let workspace = rawWorkspace;
        // on windows, we have to replace the unix-like workspace path with a windows one. /C/Users/... -> C:/Users/ ...
        // logic copied from Eclipse plugin
        // MicroclimateConnection.java#L244
        if (rawPlatform.toLowerCase() === "windows" && workspace.startsWith("/")) {
            const deviceLetter = workspace.substring(1, 2);
            workspace = deviceLetter + ":" + workspace.substring(2);
        }
        const versionNum = MCEnvironment_1.default.getVersionNumber(mcEnvData);
        // normalize namespace so it doesn't start with '/'
        const socketNS = rawSocketNS.startsWith("/") ? rawSocketNS.substring(1, rawSocketNS.length) : rawSocketNS;
        try {
            return yield ConnectionManager_1.default.instance.addConnection(mcUri, host, versionNum, socketNS, workspace);
        }
        catch (err) {
            Logger_1.default.i("New connection rejected by ConnectionManager ", err.message || err);
            throw err;
        }
    });
}
exports.onSuccessfulConnection = onSuccessfulConnection;
//# sourceMappingURL=NewConnectionCmd.js.map