"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const CommandUtil_1 = require("./CommandUtil");
const ProjectInfo = require("../microclimate/project/ProjectOverview");
const Logger_1 = require("../Logger");
const Commands_1 = require("../constants/Commands");
const Requester_1 = require("../microclimate/project/Requester");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const ToggleAutoBuildCmd_1 = require("./ToggleAutoBuildCmd");
const ToggleEnablementCmd_1 = require("./ToggleEnablementCmd");
const RequestBuildCmd_1 = require("./RequestBuildCmd");
const Resources_1 = require("../constants/Resources");
function projectOverviewCmd(project) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("projectOverviewCmd invoked");
        if (project == null) {
            const selected = yield CommandUtil_1.promptForProject();
            if (selected == null) {
                // user cancelled
                Logger_1.default.d("User cancelled project prompt");
                return;
            }
            project = selected;
        }
        const wvOptions = {
            enableScripts: true,
            retainContextWhenHidden: true,
            localResourceRoots: [vscode.Uri.file(Resources_1.default.getBaseResourcePath())]
        };
        const webPanel = vscode.window.createWebviewPanel(project.name, project.name, vscode.ViewColumn.Active, wvOptions);
        const existingPI = project.onOpenProjectInfo(webPanel);
        if (existingPI != null) {
            // Just focus them on the existing one, and do nothing more.
            existingPI.reveal();
            webPanel.dispose();
            return;
        }
        webPanel.reveal();
        webPanel.onDidDispose(() => {
            // this will dispose the webview a second time, but that seems to be fine
            project.closeProjectInfo();
        });
        const icons = project.type.icon;
        webPanel.iconPath = {
            light: vscode.Uri.file(icons.light),
            dark: vscode.Uri.file(icons.dark)
        };
        // const ed = vscode.window.activeTextEditor;
        webPanel.webview.html = ProjectInfo.generateHtml(project);
        webPanel.webview.onDidReceiveMessage((msg) => {
            Logger_1.default.d(`Got message from ProjectInfo for project ${project.name}: ${msg.type}`);
            try {
                if (msg.type === ProjectInfo.Messages.TOGGLE_AUTOBUILD) {
                    ToggleAutoBuildCmd_1.default(project);
                }
                else if (msg.type === ProjectInfo.Messages.TOGGLE_ENABLEMENT) {
                    ToggleEnablementCmd_1.default(project, !project.state.isEnabled);
                }
                else if (msg.type === ProjectInfo.Messages.BUILD) {
                    RequestBuildCmd_1.default(project);
                }
                else if (msg.type === ProjectInfo.Messages.DELETE) {
                    const deleteMsg = translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "confirmDeleteProjectMsg", { projectName: project.name });
                    const deleteBtn = translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "confirmDeleteBtn", { projectName: project.name });
                    vscode.window.showWarningMessage(deleteMsg, { modal: true }, deleteBtn)
                        .then((response) => {
                        if (response === deleteBtn) {
                            // Delete the project, then close the webview since the project is gone.
                            Requester_1.default.requestDelete(project);
                        }
                    });
                }
                else if (msg.type === ProjectInfo.Messages.OPEN) {
                    Logger_1.default.d("Got msg to open, data is ", msg.data);
                    let uri;
                    if (msg.data.type === ProjectInfo.Openable.FILE || msg.data.type === ProjectInfo.Openable.FOLDER) {
                        uri = vscode.Uri.file(msg.data.value);
                    }
                    else {
                        // default to web
                        uri = vscode.Uri.parse(msg.data.value);
                    }
                    Logger_1.default.i("The uri is:", uri);
                    const cmd = msg.data.type === ProjectInfo.Openable.FOLDER ? Commands_1.default.VSC_REVEAL_IN_OS : Commands_1.default.VSC_OPEN;
                    vscode.commands.executeCommand(cmd, uri);
                }
                else {
                    Logger_1.default.e("Received unknown event from project info webview:", msg);
                }
            }
            catch (err) {
                Logger_1.default.e("Error processing msg from WebView", err);
            }
        });
    });
}
exports.default = projectOverviewCmd;
//# sourceMappingURL=ProjectOverviewCmd.js.map