"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const TreeItemAdaptable_1 = require("../../view/TreeItemAdaptable");
const Project_1 = require("../project/Project");
const Endpoints_1 = require("../../constants/Endpoints");
const MCSocket_1 = require("./MCSocket");
const ConnectionManager_1 = require("./ConnectionManager");
const Resources_1 = require("../../constants/Resources");
const Logger_1 = require("../../Logger");
const MCLogManager_1 = require("../logs/MCLogManager");
const DebugUtils_1 = require("../project/DebugUtils");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const MCEnvironment_1 = require("./MCEnvironment");
class Connection {
    // public readonly detail?: string;
    constructor(mcUri, host, version, socketNS, workspacePath_) {
        this.mcUri = mcUri;
        this.host = host;
        this.version = version;
        this.socketNS = socketNS;
        // Is this connection CURRENTLY connected to its Microclimate instance
        this.connected = false;
        this.projects = [];
        this.needProjectUpdate = true;
        this.onConnect = () => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this} onConnect`);
            if (this.connected) {
                // we already know we're connected, nothing to do until we disconnect
                return;
            }
            if (!(yield ConnectionManager_1.default.instance.verifyReconnect(this))) {
                Logger_1.default.i(`Connection has changed on reconnect! ${this} is no longer a valid Connection`);
                // this connection gets destroyed
                return;
            }
            this.connected = true;
            Logger_1.default.d(`${this} is now connected`);
            yield this.forceUpdateProjectList();
            this.logManager.onConnectionReconnect();
            this.onChange();
        });
        this.onDisconnect = () => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this} onDisconnect`);
            if (!this.connected) {
                // we already know we're disconnected, nothing to do until we reconnect
                return;
            }
            this.connected = false;
            this.projects.forEach((p) => p.onConnectionDisconnect());
            this.projects = [];
            Logger_1.default.d(`${this} is now disconnected`);
            this.logManager.onConnectionDisconnect();
            this.onChange();
        });
        this.projectsApiUri = Endpoints_1.default.getEndpoint(this, Endpoints_1.default.PROJECTS);
        this.socket = new MCSocket_1.default(this, socketNS);
        this.logManager = new MCLogManager_1.default(this);
        this.workspacePath = vscode.Uri.file(workspacePath_);
        this.versionStr = MCEnvironment_1.default.getVersionAsString(version);
        // QuickPickItem
        this.label = this.getTreeItemLabel();
        // this.description = this.workspacePath.fsPath.toString();
        Logger_1.default.i(`Created new Connection @ ${this}, workspace ${this.workspacePath}`);
        DebugUtils_1.default.cleanDebugLaunchConfigsFor(this);
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Destroy connection " + this);
            return Promise.all([
                this.logManager.onConnectionDisconnect(),
                this.socket.destroy()
            ])
                .then(() => Promise.resolve());
        });
    }
    toString() {
        return `${this.mcUri} ${this.versionStr}`;
    }
    /**
     * Call this whenever the tree needs to be updated - ie when this connection or any of its projects changes.
     */
    onChange() {
        return __awaiter(this, void 0, void 0, function* () {
            // Log.d(`Connection ${this.mcUri} changed`);
            ConnectionManager_1.default.instance.onChange();
        });
    }
    get isConnected() {
        return this.connected;
    }
    getProjects() {
        return __awaiter(this, void 0, void 0, function* () {
            // Log.d("getProjects");
            if (!this.needProjectUpdate) {
                return this.projects;
            }
            Logger_1.default.d(`Updating projects list from ${this}`);
            const result = yield request.get(this.projectsApiUri, { json: true });
            const oldProjects = this.projects;
            this.projects = [];
            for (const projectInfo of result) {
                // This is a hard-coded exception for a Microclimate bug, where projects get stuck in the Deleting or Validating state
                // and don't go away until they're deleted from the workspace and MC is restarted.
                if (projectInfo.action === "deleting" || projectInfo.action === "validating") { // non-nls
                    Logger_1.default.e("Project is in a bad state and won't be displayed:", projectInfo);
                    continue;
                }
                let project;
                // If we already have a Project object for this project, just update it, don't make a new object
                // (since then the old object will go stale while code might still be referencing it)
                const existing = oldProjects.find((p) => p.id === projectInfo.projectID);
                if (existing != null) {
                    project = existing;
                    existing.update(projectInfo);
                    // Log.d("Reuse project " + project.name);
                }
                else {
                    project = new Project_1.default(projectInfo, this);
                    Logger_1.default.d("New project " + project.name);
                }
                this.projects.push(project);
            }
            this.needProjectUpdate = false;
            Logger_1.default.d("Done projects update");
            return this.projects;
        });
    }
    getProjectByID(projectID) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = (yield this.getProjects()).find((project) => project.id === projectID);
            if (result == null) {
                // Logger.logE(`Couldn't find project with ID ${projectID} on connection ${this}`);
            }
            return result;
        });
    }
    getChildren() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.connected) {
                const disconnectedLabel = translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "disconnectedConnectionLabel");
                // The context ID can be any truthy string.
                const disconnectedContextID = "disconnectedContextID"; // non-nls;
                const disconnectedTI = new TreeItemAdaptable_1.SimpleTreeItem(disconnectedLabel, undefined, undefined, disconnectedContextID, Resources_1.default.getIconPaths(Resources_1.default.Icons.Disconnected));
                return [disconnectedTI];
            }
            yield this.getProjects();
            // Logger.log(`Connection ${this} has ${this.projects.length} projects`);
            if (this.projects.length === 0) {
                const noProjectsTi = new TreeItemAdaptable_1.SimpleTreeItem(translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "noProjectsLabel"));
                return [noProjectsTi];
            }
            return this.projects;
        });
    }
    toTreeItem() {
        const ti = new vscode.TreeItem(this.getTreeItemLabel(), vscode.TreeItemCollapsibleState.Expanded);
        // ti.resourceUri = this.workspacePath;
        ti.tooltip = `${this.versionStr} • ${this.workspacePath.fsPath}`;
        ti.contextValue = this.getContextID();
        ti.iconPath = Resources_1.default.getIconPaths(Resources_1.default.Icons.Microclimate);
        // command run on single-click - https://github.com/Microsoft/vscode/issues/39601
        /*
        ti.command = {
            command: Project.ONCLICK_CMD_ID,
            title: "",      // non-nls
            arguments: [ti.resourceUri]
        };*/
        return ti;
    }
    getTreeItemLabel() {
        return translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "connectionLabel", { uri: this.mcUri });
    }
    getContextID() {
        if (this.connected) {
            return Connection.CONTEXT_ID_ACTIVE;
        }
        return Connection.CONTEXT_ID;
    }
    forceUpdateProjectList(wipeProjects = false) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("forceUpdateProjectList");
            if (wipeProjects) {
                Logger_1.default.d(`Connection ${this} wiping ${this.projects.length} projects`);
                // destroy the project objects because we are dereferencing them
                this.projects.forEach((p) => p.destroy());
                this.projects = [];
            }
            this.needProjectUpdate = true;
            yield this.getProjects();
        });
    }
}
Connection.CONTEXT_ID = "ext.mc.connectionItem"; // must match package.nls.json    // non-nls
Connection.CONTEXT_ID_ACTIVE = Connection.CONTEXT_ID + ".active"; // non-nls
exports.default = Connection;
//# sourceMappingURL=Connection.js.map