"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCUtil = require("../../MCUtil");
const Connection_1 = require("./Connection");
const NewConnectionCmd_1 = require("../../command/NewConnectionCmd");
const Logger_1 = require("../../Logger");
const Settings_1 = require("../../constants/Settings");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const MCEnvironment_1 = require("./MCEnvironment");
class ConnectionManager {
    constructor() {
        this._connections = [];
        this.listeners = [];
        /**
         * Call this whenever a connection is added, removed, or changed.
         */
        this.onChange = () => {
            // Logger.log(`Connection ${connection} changed`);
            this.listeners.forEach((f) => f());
        };
        const connectionInfos = ConnectionManager.loadConnections();
        Logger_1.default.i(`Loaded ${connectionInfos.length} connections from settings`);
        connectionInfos.forEach((connInfo) => {
            // Note this is done async
            // We use tryAddConnection over newConnectionCmd because it succeeds silently (but still reports failure)
            NewConnectionCmd_1.tryAddConnection(connInfo, true);
        });
    }
    static get instance() {
        return ConnectionManager._instance || (ConnectionManager._instance = new this());
    }
    get connections() {
        return this._connections;
    }
    addConnection(uri, host, mcVersion, socketNS, workspace) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.connectionExists(uri)) {
                const alreadyExists = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "connectionAlreadyExists", { uri });
                // Log.i(alreadyExists);
                throw new Error(alreadyExists);
            }
            // all validation that this connection is good must be done by this point
            const newConnection = new Connection_1.default(uri, host, mcVersion, socketNS, workspace);
            Logger_1.default.i("New Connection @ " + uri);
            this._connections.push(newConnection);
            ConnectionManager.saveConnections();
            this.onChange();
            return newConnection;
        });
    }
    removeConnection(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const indexToRemove = this.connections.indexOf(connection);
            if (indexToRemove === -1) {
                Logger_1.default.e(`Request to remove connection ${connection} but it doesn't exist!`);
                return false;
            }
            connection.destroy();
            this.connections.splice(indexToRemove, 1);
            Logger_1.default.i("Removed connection", connection);
            ConnectionManager.saveConnections();
            this.onChange();
            return true;
        });
    }
    /**
     * To be called on connection reconnect. Hits the environment endpoint at the given existing Connection's URI,
     * and returns if the response data matches the existing Connection.
     *
     * The given Connection will be destroyed if the data does not match (ie, this function returns `false`),
     * and thus must not do anything further.
     */
    verifyReconnect(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Verifying reconnect at " + connection.mcUri);
            let tries = 0;
            let newEnvData;
            // Sometimes this can execute before Portal is ready, resulting in a 404.
            while (newEnvData == null && tries < 10) {
                tries++;
                try {
                    newEnvData = yield MCEnvironment_1.default.getEnvData(connection.mcUri);
                }
                catch (err) {
                    // wait briefly before trying again
                    yield new Promise((resolve) => setTimeout(resolve, 250));
                }
            }
            if (newEnvData == null) {
                // I don't think this will ever happen
                Logger_1.default.e("Couldn't get a good response from environment endpoint " + connection.mcUri);
                vscode.window.showErrorMessage(translator_1.default.t(StringNamespaces_1.default.DEFAULT, "failedToReconnect", { uri: connection.mcUri }));
                yield this.removeConnection(connection);
                return false;
            }
            if (MCEnvironment_1.default.envMatches(connection, newEnvData)) {
                // it's the same instance, so we don't have to do anything
                return true;
            }
            else {
                Logger_1.default.d("Microclimate instance changed on reconnect!");
                yield this.removeConnection(connection);
                // will also add the new Connection to this ConnectionManager
                const newConnection = yield NewConnectionCmd_1.newConnectionCmd(MCUtil.getConnInfoFrom(connection.mcUri));
                if (newConnection == null) {
                    // should never happen
                    Logger_1.default.e("Failed to create new connection after verifyReconnect failure");
                    return false;
                }
                const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "versionChanged", { uri: connection.mcUri, oldVersion: connection.versionStr, newVersion: newConnection.versionStr });
                vscode.window.showInformationMessage(msg);
                return false;
            }
        });
    }
    connectionExists(uri) {
        return this._connections.some((conn) => {
            return conn.mcUri.toString() === uri.toString();
        });
    }
    static loadConnections() {
        const globalState = global.extGlobalState;
        const loaded = globalState.get(Settings_1.default.CONNECTIONS_KEY) || [];
        return loaded;
    }
    static saveConnections() {
        return __awaiter(this, void 0, void 0, function* () {
            // We save IConnectionInfo objects since they are simpler and more readable than VSCode URIs.
            // This will likely change with ICP support since we would then have to store protocol too.
            const connectionInfos = ConnectionManager.instance.connections
                .map((connection) => MCUtil.getConnInfoFrom(connection.mcUri));
            Logger_1.default.i("Saving connections", connectionInfos);
            try {
                const globalState = global.extGlobalState;
                // connectionInfos must not contain cyclic references (ie, JSON.stringify succeeds)
                yield globalState.update(Settings_1.default.CONNECTIONS_KEY, connectionInfos);
            }
            catch (err) {
                const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "errorSavingConnections", { err: err.toString() });
                Logger_1.default.e(msg, err);
                vscode.window.showErrorMessage(msg);
            }
        });
    }
    /**
     * Pass this a function to call whenever a connection is added, removed, or changed,
     * eg to trigger a tree update in the UI.
     * Test-friendly.
     */
    addOnChangeListener(callback) {
        Logger_1.default.i("Adding onChangeListener " + callback.name);
        this.listeners.push(callback);
    }
}
exports.default = ConnectionManager;
//# sourceMappingURL=ConnectionManager.js.map