"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request-promise-native");
const reqErrors = require("request-promise-native/errors");
const Logger_1 = require("../../Logger");
const Endpoints_1 = require("../../constants/Endpoints");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const STRING_NS = StringNamespaces_1.default.CMD_NEW_CONNECTION;
var MCEnvironment;
(function (MCEnvironment) {
    function getEnvData(mcUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const envUri = mcUri.with({ path: Endpoints_1.default.ENVIRONMENT });
            const connectTimeout = 2500;
            try {
                const result = yield request.get(envUri.toString(), { json: true, timeout: connectTimeout });
                return result;
            }
            catch (err) {
                Logger_1.default.i(`Connection ENV Request fail - ${err}`);
                if (err instanceof reqErrors.RequestError) {
                    throw new Error(translator_1.default.t(STRING_NS, "connectFailed", { uri: mcUri }));
                }
                throw err;
            }
        });
    }
    MCEnvironment.getEnvData = getEnvData;
    // microclimate_version and workspace_location were both added in Microclimate 18.09
    // Portal Restart API improvement was added in 18.11
    MCEnvironment.REQUIRED_VERSION_STR = "18.12"; // non-nls
    const REQUIRED_VERSION = 1812;
    const INTERNAL_BUILD_RX = /^\d{4}_M\d+_[EI]/;
    /**
     * Parses a version number out of the given env data. If it's a development build, returns Number.MAX_SAFE_INTEGER.
     *
     * **Throws an error** if the version is not supported.
     */
    function getVersionNumber(envData) {
        const rawVersion = envData.microclimate_version;
        if (rawVersion === "latest") { // non-nls
            // This means it's being hosted by an internal MC dev.
            // There's nothing we can do here but assume they have all the features we need.
            Logger_1.default.i("Dev version of Microclimate");
            return Number.MAX_SAFE_INTEGER;
        }
        else if (rawVersion.match(INTERNAL_BUILD_RX) != null) {
            Logger_1.default.i("Internal build of Microclimate");
            return Number.MAX_SAFE_INTEGER;
        }
        else {
            const versionNum = Number(rawVersion);
            if (isNaN(versionNum)) {
                Logger_1.default.e("Couldn't convert provided version to Number, version is: " + rawVersion);
                throw new Error(translator_1.default.t(STRING_NS, "versionNotRecognized", { rawVersion: rawVersion, requiredVersion: MCEnvironment.REQUIRED_VERSION_STR }));
            }
            else if (versionNum < REQUIRED_VERSION) {
                Logger_1.default.e(`Microclimate version ${versionNum} is too old.`);
                throw new Error(translator_1.default.t(STRING_NS, "versionTooOld", { rawVersion: rawVersion, requiredVersion: MCEnvironment.REQUIRED_VERSION_STR }));
            }
            return versionNum;
        }
    }
    MCEnvironment.getVersionNumber = getVersionNumber;
    function getVersionAsString(versionNum) {
        if (versionNum === Number.MAX_SAFE_INTEGER) {
            return "latest";
        }
        else {
            const year = Math.floor(versionNum / 100);
            const month = versionNum % 100;
            return `${year}.${month < 10 ? "0" + month : month}`;
        }
    }
    MCEnvironment.getVersionAsString = getVersionAsString;
    /**
     * @returns If the given Connection matches the given environment data for fields the tools are interested in.
     */
    function envMatches(connection, envData) {
        let newVersionNumber;
        try {
            newVersionNumber = getVersionNumber(envData);
        }
        catch (err) {
            Logger_1.default.w(err);
            return false;
        }
        return connection.version === newVersionNumber;
        // should check workspace too, but need to consider platform when comparing paths
        // more to check once we support ICP
        // envData.user_string
    }
    MCEnvironment.envMatches = envMatches;
})(MCEnvironment || (MCEnvironment = {}));
exports.default = MCEnvironment;
//# sourceMappingURL=MCEnvironment.js.map