"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const io = require("socket.io-client");
const Logger_1 = require("../../Logger");
const Validator_1 = require("../project/Validator");
const SocketEvents_1 = require("./SocketEvents");
/**
 * Receives and reacts to socket events from Portal
 *
 * Each Connection has exactly one socket
 */
class MCSocket {
    /**
     * Create a SocketIO connection to the server at the given URI.
     * Can throw an error.
     *
     * @param namespace - Socket namespace. Must not start with a slash. Can be the empty string.
     */
    constructor(connection, namespace) {
        this.connection = connection;
        this.onProjectCreated = (payload) => __awaiter(this, void 0, void 0, function* () {
            yield this.connection.forceUpdateProjectList();
            const newProject = yield this.connection.getProjectByID(payload.projectID);
            if (newProject == null) {
                Logger_1.default.e(`Project ${payload.projectID} was created but not available after a refresh`);
            }
            else {
                Logger_1.default.i(`Project ${newProject.name} has been created`);
            }
        });
        this.onProjectStatusChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("onProjectStatusChanged", payload);
            // I don't see any reason why these should be handled differently
            this.onProjectChanged(payload);
        });
        this.onProjectChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("onProjectChanged", payload);
            // Log.d(`PROJECT CHANGED name=${payload.name} appState=${payload.appStatus} ` +
            // `buildState=${payload.buildStatus} startMode=${payload.startMode}`);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            project.update(payload);
        });
        this.onProjectClosed = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            yield project.clearValidationErrors();
            this.onProjectChanged(payload);
        });
        this.onProjectDeleted = (payload) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("PROJECT DELETED", payload.projectID);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            yield project.onDelete();
            this.connection.forceUpdateProjectList();
        });
        this.onProjectRestarted = (payload) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("PROJECT RESTARTED", payload);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            project.onRestartEvent(payload);
        });
        this.onContainerLogs = (payload) => __awaiter(this, void 0, void 0, function* () {
            const projectID = payload.projectID;
            // const projectName = payload.projectName;
            const logContents = payload.logs;
            const log = this.connection.logManager.getAppLog(projectID);
            if (log != null) {
                log.update(logContents);
            }
        });
        this.onProjectValidated = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            if (payload.validationResults != null) {
                Validator_1.default.validate(project, payload.validationResults);
            }
            else {
                Logger_1.default.e("Microclimate didn't send result with validation event");
            }
        });
        this.getProject = (payload) => __awaiter(this, void 0, void 0, function* () {
            const projectID = payload.projectID;
            if (projectID == null) {
                // Should never happen
                Logger_1.default.e("No projectID in socket event!", payload);
                return undefined;
            }
            return this.connection.getProjectByID(projectID);
        });
        this.uri = connection.mcUri.toString();
        if (namespace) {
            if (!this.uri.endsWith("/")) {
                this.uri += "/";
            }
            this.uri += namespace;
        }
        Logger_1.default.i("Creating MCSocket for URI", this.uri);
        const options = {
        // rejectUnauthorized:
        };
        this.socket = io(this.uri, options);
        this.socket.connect();
        this.socket
            .on("connect", this.connection.onConnect) // non-nls
            .on("disconnect", this.connection.onDisconnect) // non-nls
            .on(SocketEvents_1.default.Types.PROJECT_CREATED, this.onProjectCreated)
            .on(SocketEvents_1.default.Types.PROJECT_CHANGED, this.onProjectChanged)
            .on(SocketEvents_1.default.Types.PROJECT_STATUS_CHANGED, this.onProjectStatusChanged)
            .on(SocketEvents_1.default.Types.PROJECT_CLOSED, this.onProjectClosed)
            .on(SocketEvents_1.default.Types.PROJECT_DELETION, this.onProjectDeleted)
            .on(SocketEvents_1.default.Types.PROJECT_RESTART_RESULT, this.onProjectRestarted)
            .on(SocketEvents_1.default.Types.CONTAINER_LOGS, this.onContainerLogs)
            .on(SocketEvents_1.default.Types.PROJECT_VALIDATED, this.onProjectValidated);
    }
    /**
     * This MUST be called when the connection is removed.
     * If there are multiple sockets listening on the same connection,
     * the callbacks will be fired multiple times for the same event, which will lead to serious misbehaviour.
     */
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            this.socket.disconnect();
        });
    }
    toString() {
        return "MCSocket @ " + this.uri; // not displayed to user        // non-nls
    }
    toJSON() {
        return this.toString();
    }
}
exports.default = MCSocket;
//# sourceMappingURL=MCSocket.js.map