"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const Logger_1 = require("../../Logger");
const Endpoints_1 = require("../../constants/Endpoints");
const MCLog_1 = require("./MCLog");
const translator_1 = require("../../constants/strings/translator");
class BuildLog extends MCLog_1.default {
    constructor(connection, projectID, projectName, managerMap) {
        super(projectID, projectName, MCLog_1.default.LogTypes.BUILD, managerMap, translator_1.default.t(MCLog_1.default.STRING_NS, "waitingForBuildLogs", { projectName }));
        this.connection = connection;
        this.projectID = projectID;
        this.projectName = projectName;
        this.lastUpdated = new Date(0);
        this.update = () => __awaiter(this, void 0, void 0, function* () {
            if (!this.doUpdate) {
                Logger_1.default.e("Update was invoked on an buildLog with doUpdate=false, this should never happen!");
            }
            const buildLogUrl = Endpoints_1.default.getProjectEndpoint(this.connection, this.projectID, Endpoints_1.default.BUILD_LOG);
            try {
                const getResult = yield request.get(buildLogUrl, { resolveWithFullResponse: true });
                const lastModifiedStr = getResult.headers[BuildLog.LAST_UPDATED_HEADER];
                const lastModified = new Date(Number(lastModifiedStr));
                // Logger.log("buildlog-lastModified", lastModifiedStr, lastModified);
                if (lastModified == null || lastModified > this.lastUpdated) {
                    Logger_1.default.d("Updating " + this.outputChannel.name);
                    this.lastUpdated = new Date(lastModified);
                    // The build log doesn't get appended to, it's always totally new
                    this.outputChannel.clear();
                    this.outputChannel.appendLine(getResult.body);
                    this.onChange();
                }
                /*
                else {
                    Log.d(`${this.outputChannel.name} hasn't changed`);
                }*/
            }
            catch (err) {
                Logger_1.default.e(err);
                if (err.statusCode === 404) {
                    // The project got deleted or disabled
                    return this.stopUpdating(false);
                }
                // Allow the user to kill this log so it doesn't spam them with error messages if there's a network problem or something.
                const stopUpdatingBtn = translator_1.default.t(MCLog_1.default.STRING_NS, "stopUpdatingBtn");
                vscode.window.showErrorMessage(translator_1.default.t(MCLog_1.default.STRING_NS, "errUpdatingBuildLog", { err: err.toString() }), stopUpdatingBtn)
                    .then((btn) => {
                    if (btn === stopUpdatingBtn) {
                        this.stopUpdating(false);
                    }
                });
            }
        });
        this.update();
        this.timer = setInterval(this.update, BuildLog.UPDATE_INTERVAL);
    }
    stopUpdating(connectionLost = true) {
        const _super = Object.create(null, {
            stopUpdating: { get: () => super.stopUpdating }
        });
        return __awaiter(this, void 0, void 0, function* () {
            clearInterval(this.timer);
            _super.stopUpdating.call(this, connectionLost);
        });
    }
    showOutputChannel() {
        const _super = Object.create(null, {
            showOutputChannel: { get: () => super.showOutputChannel }
        });
        return __awaiter(this, void 0, void 0, function* () {
            this.update();
            _super.showOutputChannel.call(this);
        });
    }
}
BuildLog.UPDATE_INTERVAL = 5000;
BuildLog.LAST_UPDATED_HEADER = "build-log-last-modified"; // non-nls
exports.default = BuildLog;
//# sourceMappingURL=BuildLog.js.map