"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Logger_1 = require("../../Logger");
const Settings_1 = require("../../constants/Settings");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const translator_1 = require("../../constants/strings/translator");
/**
 * Type to capture common functionality between AppLogs and BuildLogs
 */
class MCLog {
    constructor(projectID, projectName, logType, managerMap, initialMsg) {
        this.projectID = projectID;
        this.projectName = projectName;
        this.logType = logType;
        this.managerMap = managerMap;
        this.doUpdate = true;
        const outputChannelName = translator_1.default.t(MCLog.STRING_NS, "logName", { projectName, logType: logType.toString() });
        this.outputChannel = vscode.window.createOutputChannel(outputChannelName);
        this.outputChannel.appendLine(initialMsg);
        // quickPickItem
        this.label = this.outputChannel.name;
    }
    showOutputChannel() {
        return __awaiter(this, void 0, void 0, function* () {
            this.outputChannel.show(true);
        });
    }
    /**
     * Print a message to the bottom of the outputstream that this log won't be updated
     * until the 'show log' command is run again, and stop updating this log.
     *
     * @param connectionLost -
     *  A slightly different message is displayed if the updates were cancelled because we lost the MC connection.
     *  It is possible for a user to manually stop the updates of a build log, in which case this should be false.
     */
    stopUpdating(connectionLost = true) {
        if (!this.doUpdate) {
            Logger_1.default.d("Already stopped updating log " + this.outputChannel.name);
            return;
        }
        Logger_1.default.d("stopUpdating log " + this.outputChannel.name);
        // prevents printing the 'no more updates' message more than once
        this.doUpdate = false;
        const msgKey = connectionLost ? "logNotUpdatingNoConnection" : "logNotUpdatingOther"; // non-nls
        const msg = translator_1.default.t(MCLog.STRING_NS, msgKey, { logType: this.logType.toString().toLowerCase() });
        this.outputChannel.appendLine("\n" + msg); // non-nls
    }
    /**
     * Disposes of this log's OutputChannel.
     * Also removes it from the owner manager's log map, which is a nasty coupling I will revisit!
     */
    destroy() {
        Logger_1.default.d("Destroy log " + this.outputChannel.name);
        this.stopUpdating();
        this.outputChannel.dispose();
        if (this.managerMap.has(this.projectID)) {
            this.managerMap.delete(this.projectID);
        }
    }
    /**
     * Call this whenever this log gets updated. It will show the log if the user has that preference set.
     */
    onChange() {
        const setting = this.logType === MCLog.LogTypes.APP ? Settings_1.default.OPEN_ON_CHANGE_APP : Settings_1.default.OPEN_ON_CHANGE_BUILD;
        const showOnChange = vscode.workspace.getConfiguration(Settings_1.default.CONFIG_SECTION)
            .get(setting, false);
        if (showOnChange) {
            this.showOutputChannel();
        }
    }
}
MCLog.STRING_NS = StringNamespaces_1.default.LOGS;
exports.MCLog = MCLog;
(function (MCLog) {
    let LogTypes;
    (function (LogTypes) {
        LogTypes["BUILD"] = "Build";
        LogTypes["APP"] = "App";
    })(LogTypes = MCLog.LogTypes || (MCLog.LogTypes = {}));
})(MCLog = exports.MCLog || (exports.MCLog = {}));
exports.default = MCLog;
//# sourceMappingURL=MCLog.js.map