"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AppLog_1 = require("./AppLog");
const BuildLog_1 = require("./BuildLog");
// Not to be confused with the other log classes :)
const Logger_1 = require("../../Logger");
/**
 * Contains info about app and build logs for a given Connection.
 * Manages creating logs, and handle properly Microclimate going down while we have logs open.
 */
class MCLogManager {
    constructor(connection) {
        this.connection = connection;
        // Maps projectIDs to Log instances
        // TODO revist this. I think these can be combined into one array now, but will require an overhaul of this class.
        this.appLogMap = new Map();
        this.buildLogMap = new Map();
    }
    /**
     * Returns an array of all open logs for this Connection (aka this logManager)
     */
    getAllOpenLogs() {
        let openLogs = [];
        openLogs = openLogs.concat(Array.from(this.appLogMap.values()));
        openLogs = openLogs.concat(Array.from(this.buildLogMap.values()));
        return openLogs;
    }
    getOrCreateAppLog(projectID, projectName) {
        let appLog = this.appLogMap.get(projectID);
        if (appLog == null) {
            Logger_1.default.i("Creating app log for " + projectName);
            // we have to create it
            appLog = new AppLog_1.default(projectID, projectName, this.appLogMap);
            this.appLogMap.set(projectID, appLog);
        }
        return appLog;
    }
    getAppLog(projectID) {
        return this.appLogMap.get(projectID);
    }
    getOrCreateBuildLog(projectID, projectName) {
        let buildLog = this.buildLogMap.get(projectID);
        if (buildLog == null) {
            Logger_1.default.i("Creating build log for " + projectName);
            // we have to create it
            buildLog = new BuildLog_1.default(this.connection, projectID, projectName, this.buildLogMap);
            this.buildLogMap.set(projectID, buildLog);
        }
        return buildLog;
    }
    getBuildLog(projectID) {
        return this.buildLogMap.get(projectID);
    }
    destroyLogsForProject(projectID) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Destroying logs for project " + projectID);
            const appLog = this.getAppLog(projectID);
            if (appLog != null) {
                appLog.destroy();
            }
            const buildLog = this.getBuildLog(projectID);
            if (buildLog != null) {
                buildLog.destroy();
            }
        });
    }
    /**
     * When a connection dies, we have to stop updating its logs, but we should keep the logs visible.
     */
    onConnectionDisconnect() {
        // Log.d(`LogManager for ${this.connection.mcUri} onDisconnect`);
        this.appLogMap.forEach((log) => {
            log.stopUpdating();
        });
        this.buildLogMap.forEach((log) => {
            log.stopUpdating();
        });
    }
    /**
     * When a connection is reset, we have to wipe all the logs since the old logs are no longer valid.
     * The easiest way to is destroy them and start from scratch.
     */
    onConnectionReconnect() {
        // Log.d(`LogManager for ${this.connection.mcUri} onReconnect`);
        const oldAppLogs = new Map(this.appLogMap);
        this.appLogMap.clear();
        oldAppLogs.forEach((log) => {
            log.destroy();
            this.getOrCreateAppLog(log.projectID, log.projectName);
        });
        const oldBuildLogs = new Map(this.buildLogMap);
        this.buildLogMap.clear();
        oldBuildLogs.forEach((log) => {
            log.destroy();
            this.getOrCreateBuildLog(log.projectID, log.projectName);
        });
    }
}
exports.default = MCLogManager;
//# sourceMappingURL=MCLogManager.js.map