"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCUtil = require("../../MCUtil");
const ProjectState_1 = require("./ProjectState");
const ProjectType_1 = require("./ProjectType");
const Logger_1 = require("../../Logger");
const Commands_1 = require("../../constants/Commands");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const ProjectOverview_1 = require("./ProjectOverview");
const ProjectContextID_1 = require("./ProjectContextID");
const ProjectPendingRestart_1 = require("./ProjectPendingRestart");
const StartModes_1 = require("../../constants/StartModes");
const SocketEvents_1 = require("../connection/SocketEvents");
const DebugUtils_1 = require("./DebugUtils");
const STRING_NS = StringNamespaces_1.default.PROJECT;
class Project {
    constructor(projectInfo, connection) {
        this.connection = connection;
        // in MS
        this.RESTART_TIMEOUT = 180 * 1000;
        /**
         * Set this project's status based on the project info event payload passed.
         * This includes checking the appStatus, buildStatus, buildStatusDetail, and startMode.
         * Also updates the appPort and debugPort.
         *
         * Also signals the ConnectionManager change listener
         */
        this.update = (projectInfo, isRestart = false) => {
            if (projectInfo.projectID !== this.id) {
                // shouldn't happen, but just in case
                Logger_1.default.e(`Project ${this.id} received status update request for wrong project ${projectInfo.projectID}`);
                // return the old state
                return this._state;
            }
            // Whether or not this update call has changed the project such that we have to update the UI.
            let changed = false;
            if (!isRestart) {
                // Ignore these if it's a restart because the restart event won't have them
                changed = this.setContainerID(projectInfo.containerId) || changed;
                changed = this.setLastBuild(projectInfo.lastbuild) || changed;
                // appImageLastBuild is a string
                changed = this.setLastImgBuild(Number(projectInfo.appImageLastBuild)) || changed;
                changed = this.setAutoBuild(projectInfo.autoBuild) || changed;
            }
            // note oldState can be null if this is the first time update is being invoked.
            const oldState = this._state;
            this._state = new ProjectState_1.default(projectInfo, oldState != null ? oldState : undefined);
            if (!this._state.equals(oldState)) {
                changed = true;
                const startModeMsg = projectInfo.startMode == null ? "" : `, startMode=${projectInfo.startMode}`;
                Logger_1.default.d(`${this.name} went from ${oldState} to ${this._state}${startModeMsg}`);
            }
            const ports = projectInfo.ports;
            if (ports != null) {
                changed = this.setAppPort(ports.exposedPort) || changed;
                changed = this.setDebugPort(ports.exposedDebugPort) || changed;
            }
            else if (this._state.isStarted) {
                Logger_1.default.e("No ports were provided for an app that is supposed to be started");
            }
            if (this.pendingRestart != null) {
                this.pendingRestart.onStateChange(this.state.appState);
            }
            // Logger.log(`${this.name} has a new status:`, this._state);
            if (changed) {
                // Log.d(`${this.name} has changed`);
                this.connection.onChange();
                this.tryRefreshProjectInfoPage();
            }
            return this._state;
        };
        Logger_1.default.d("Creating project from info:", projectInfo);
        this.name = projectInfo.name;
        this.id = projectInfo.projectID;
        // should use projectType not buildType but it's missing sometimes
        // https://github.ibm.com/dev-ex/portal/issues/520
        this.type = new ProjectType_1.default(projectInfo.buildType, projectInfo.language);
        this.localPath = vscode.Uri.file(MCUtil.appendPathWithoutDupe(connection.workspacePath.fsPath, vscode.Uri.file(projectInfo.locOnDisk).fsPath));
        this.contextRoot = projectInfo.contextroot || ""; // non-nls
        // These will be overridden by the call to update(), but we set them here too so the compiler can see they're always set.
        this._autoBuildEnabled = projectInfo.autoBuild;
        // lastbuild is a number
        this._lastBuild = new Date(projectInfo.lastbuild);
        // appImageLastBuild is a string
        this._lastImgBuild = new Date(Number(projectInfo.appImgLastBuild));
        this._state = this.update(projectInfo);
        // QuickPickItem
        this.label = translator_1.default.t(STRING_NS, "quickPickLabel", { projectName: this.name, projectType: this.type.type });
        // this.detail = this.id;
        Logger_1.default.i(`Created project ${this.name}:`, this);
    }
    getChildren() {
        // Projects have no children.
        return [];
    }
    toTreeItem() {
        const ti = new vscode.TreeItem(translator_1.default.t(StringNamespaces_1.default.TREEVIEW, "projectLabel", { projectName: this.name, state: this.state.toString() }), vscode.TreeItemCollapsibleState.None);
        // ti.resourceUri = this.localPath;
        ti.tooltip = this.state.toString();
        // There are different context menu actions available to enabled or disabled or debugging projects
        ti.contextValue = ProjectContextID_1.default(this);
        ti.iconPath = this.type.icon;
        // command run on single-click (or double click - depends on a user setting - https://github.com/Microsoft/vscode/issues/39601)
        // Focuses on this project in the middle of the explorer view. Has no effect if the project is not in the current workspace.
        ti.command = {
            command: Commands_1.default.VSC_REVEAL_EXPLORER,
            title: "",
            arguments: [this.localPath]
        };
        return ti;
    }
    // description used by QuickPickItem
    get description() {
        const appUrl = this.appBaseUrl;
        if (appUrl != null) {
            return appUrl.toString();
        }
        else {
            return translator_1.default.t(STRING_NS, "quickPickNotRunning");
        }
    }
    onConnectionDisconnect() {
        if (this.pendingRestart != null) {
            this.pendingRestart.onConnectionDisconnect();
        }
    }
    get isRestarting() {
        return this.pendingRestart != null;
    }
    doRestart(mode) {
        if (this.pendingRestart != null) {
            // should be prevented by the RestartProjectCommand
            Logger_1.default.e(this.name + ": doRestart called when already restarting");
            return false;
        }
        this.pendingRestart = new ProjectPendingRestart_1.default(this, mode, this.RESTART_TIMEOUT);
        return true;
    }
    onRestartFinish() {
        Logger_1.default.d(this.name + ": onRestartFinish");
        this.pendingRestart = undefined;
    }
    /**
     * Validate the restart event. If it succeeded, update ports.
     * Notifies the pendingRestart.
     */
    onRestartEvent(event) {
        let success;
        let errMsg;
        if (this.pendingRestart == null) {
            Logger_1.default.e(this.name + ": received restart event without a pending restart", event);
            return;
        }
        if (SocketEvents_1.default.STATUS_SUCCESS !== event.status) {
            Logger_1.default.e(`${this.name}: Restart failed, response is`, event);
            errMsg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { thisName: this.name });
            if (event.errorMsg != null) {
                errMsg = event.errorMsg;
            }
            success = false;
        }
        else if (event.ports == null || event.startMode == null || !StartModes_1.default.allStartModes().includes(event.startMode)) {
            // If the status is "success" (as we just checked), these must all be set and valid
            errMsg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { thisName: this.name });
            Logger_1.default.e(errMsg + ", payload:", event);
            success = false;
        }
        else {
            Logger_1.default.d("Restart event is valid");
            this.setAppPort(event.ports.exposedPort);
            this.setDebugPort(event.ports.exposedDebugPort);
            this.tryRefreshProjectInfoPage();
            success = true;
        }
        this.pendingRestart.onReceiveRestartEvent(success, errMsg);
    }
    /**
     * Call when this project object is dereferenced/removed from the Connection, or deleted in Microclimate
     */
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Deleting project " + this.name);
            yield this.clearValidationErrors();
            yield this.connection.logManager.destroyLogsForProject(this.id);
            this.closeProjectInfo();
        });
    }
    onDelete() {
        return __awaiter(this, void 0, void 0, function* () {
            yield DebugUtils_1.default.removeDebugLaunchConfigFor(this);
            vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "onDeletion", { projectName: this.name }));
            yield this.destroy();
        });
    }
    /**
     * Clear all diagnostics for this project's path
     */
    clearValidationErrors() {
        return __awaiter(this, void 0, void 0, function* () {
            Project.diagnostics.delete(this.localPath);
        });
    }
    /**
     * To be called when the user tries to open this project's Project Info page.
     *
     * If the user already has a Project Info page open for this project, returns the existing page.
     * In this case, the webview should be re-revealed, but a new one should not be created.
     * If the user does not already have an info page open for this project, returns undefined,
     * and sets the given webview to be this project's project info panel.
     */
    onOpenProjectInfo(wvPanel) {
        if (this.activeProjectInfo != null) {
            return this.activeProjectInfo;
        }
        Logger_1.default.d(`Info opened for project ${this.name}`);
        this.activeProjectInfo = wvPanel;
        return undefined;
    }
    closeProjectInfo() {
        if (this.activeProjectInfo != null) {
            this.activeProjectInfo.dispose();
            this.activeProjectInfo = undefined;
        }
    }
    tryRefreshProjectInfoPage() {
        if (this.activeProjectInfo != null) {
            Logger_1.default.d("Refreshing projectinfo");
            ProjectOverview_1.refreshProjectOverview(this.activeProjectInfo, this);
        }
    }
    get containerID() {
        return this._containerID;
    }
    get appPort() {
        return this._appPort;
    }
    get debugPort() {
        return this._debugPort;
    }
    get autoBuildEnabled() {
        return this._autoBuildEnabled;
    }
    get state() {
        return this._state;
    }
    get appBaseUrl() {
        if (this._appPort == null) {
            // app is stopped, disabled, etc.
            return undefined;
        }
        return this.connection.mcUri.with({
            authority: `${this.connection.host}:${this._appPort}`,
            path: this.contextRoot
        });
    }
    get debugUrl() {
        if (this._debugPort == null) {
            return undefined;
        }
        return this.connection.host + ":" + this._debugPort; // non-nls
    }
    get lastBuild() {
        return this._lastBuild;
    }
    get lastImgBuild() {
        return this._lastImgBuild;
    }
    get hasContextRoot() {
        return this.contextRoot != null && this.contextRoot.length > 0 && this.contextRoot !== "/";
    }
    /**
     *
     * @return If this project's app port was changed.
     */
    setAppPort(newAppPort) {
        newAppPort = Number(newAppPort);
        if (isNaN(newAppPort)) {
            // Should happen when the app stops.
            if (this._appPort != null) {
                Logger_1.default.d("Unset app port for " + this.name);
                this._appPort = undefined;
                return true;
            }
        }
        else if (!MCUtil.isGoodPort(newAppPort)) {
            Logger_1.default.w(`Invalid app port ${newAppPort} given to project ${this.name}, ignoring it`);
            return false;
        }
        else if (this._appPort !== newAppPort) {
            this._appPort = newAppPort;
            Logger_1.default.d(`New app port for ${this.name} is ${newAppPort}`);
            return true;
        }
        return false;
    }
    /**
     *
     * @return If this project's debug port was changed.
     */
    setDebugPort(newDebugPort) {
        newDebugPort = Number(newDebugPort);
        if (isNaN(newDebugPort)) {
            // Should happen when the app stops or exits debug mode
            if (this._debugPort != null) {
                Logger_1.default.d("Unset debug port for " + this.name);
                this._debugPort = undefined;
                return true;
            }
        }
        else if (!MCUtil.isGoodPort(newDebugPort)) {
            Logger_1.default.w(`Invalid debug port ${newDebugPort} given to project ${this.name}, ignoring it`);
            return false;
        }
        else if (this._debugPort !== newDebugPort) {
            this._debugPort = newDebugPort;
            Logger_1.default.d(`New debug port for ${this.name} is ${newDebugPort}`);
            return true;
        }
        return false;
    }
    setContainerID(newContainerID) {
        const oldContainerID = this._containerID;
        this._containerID = newContainerID;
        const changed = this._containerID !== oldContainerID;
        if (changed) {
            const asStr = this._containerID == null ? "undefined" : this._containerID.substring(0, 8);
            if (asStr.length === 0) {
                Logger_1.default.w(`Empty containerID for ${this.name}`);
            }
            Logger_1.default.d(`New containerID for ${this.name} is ${asStr}`);
        }
        return changed;
    }
    setLastBuild(newLastBuild) {
        if (newLastBuild == null) {
            return false;
        }
        const oldlastBuild = this._lastBuild;
        this._lastBuild = new Date(newLastBuild);
        const changed = this._lastBuild !== oldlastBuild;
        if (changed) {
            // Log.d(`New lastBuild for ${this.name} is ${this._lastBuild}`);
        }
        return changed;
    }
    setLastImgBuild(newLastImgBuild) {
        if (newLastImgBuild == null) {
            return false;
        }
        const oldlastImgBuild = this._lastImgBuild;
        this._lastImgBuild = new Date(newLastImgBuild);
        const changed = this._lastImgBuild !== oldlastImgBuild;
        if (changed) {
            // Log.d(`New lastImgBuild for ${this.name} is ${this._lastImgBuild}`);
        }
        return changed;
    }
    setAutoBuild(newAutoBuild) {
        if (newAutoBuild == null) {
            return false;
        }
        const oldAutoBuild = this._autoBuildEnabled;
        this._autoBuildEnabled = newAutoBuild;
        const changed = this._autoBuildEnabled !== oldAutoBuild;
        if (changed) {
            this.tryRefreshProjectInfoPage();
            Logger_1.default.d(`New autoBuild for ${this.name} is ${this._autoBuildEnabled}`);
            // since setAutoBuild can be called outside of update(), we have to trigger the tree update here too
            this.connection.onChange();
        }
        return changed;
    }
}
Project.diagnostics = vscode.languages.createDiagnosticCollection("Microclimate");
exports.default = Project;
//# sourceMappingURL=Project.js.map