"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Resources_1 = require("../../constants/Resources");
const MCUtil = require("../../MCUtil");
// This file does have a bunch of strings that should be translated,
// but the stringfinder is not smart enough to pick them out from the regular html strings. So, do this file by hand.
// non-nls-file
/**
 * These are the messages the WebView can send back to its creator in ProjectInfoCmd
 */
var Messages;
(function (Messages) {
    Messages["BUILD"] = "build";
    Messages["TOGGLE_AUTOBUILD"] = "toggleAutoBuild";
    Messages["OPEN"] = "open";
    Messages["DELETE"] = "delete";
    Messages["TOGGLE_ENABLEMENT"] = "toggleEnablement";
})(Messages = exports.Messages || (exports.Messages = {}));
const resourceScheme = "vscode-resource:";
var Openable;
(function (Openable) {
    Openable["WEB"] = "web";
    Openable["FILE"] = "file";
    Openable["FOLDER"] = "folder";
})(Openable = exports.Openable || (exports.Openable = {}));
function refreshProjectOverview(webviewPanel, project) {
    webviewPanel.webview.html = generateHtml(project);
}
exports.refreshProjectOverview = refreshProjectOverview;
function generateHtml(project) {
    const emptyRow = `
    <tr>
        <td>&nbsp;</td>
        <td>&nbsp;</td>
    </tr>
    `;
    return `
        <!DOCTYPE html>

        <html>
        <head>
            <meta charset="UTF-8">
            <!--meta http-equiv="Content-Security-Policy" content="default-src 'self' ;"-->
            <meta name="viewport" content="width=device-width, initial-scale=1.0">

            <link rel="stylesheet" href="${getStylesheetPath()}"/>
        </head>
        <body>

        <div id="main">
            <div id="top-section">
                <img id="mc-icon" width="30px" src="${getMCIconPath()}"/>
                <h2>Project ${project.name}</h2>
                <input id="build-btn" type="button" value="Build"
                    onclick="${project.state.isEnabled ? `sendMsg('${Messages.BUILD}')` : ""}"
                    class="btn ${project.state.isEnabled ? "" : "btn-disabled"}"/>
            </div>

            <table id="project-info-table">
                <!--${buildRow("Name", project.name)}-->
                ${buildRow("Type", project.type.toString())}
                <!--${buildRow("Microclimate URL", project.connection.toString())}-->
                ${buildRow("Project ID", project.id)}
                ${buildRow("Container ID", getNonNull(project.containerID, "Not available", 32))}
                ${buildRow("Location on Disk", project.localPath.fsPath, Openable.FOLDER)}
                <tr>
                    <td class="info-label">Auto build:</td>
                    <td>
                        <input id="auto-build-toggle" type="checkbox" class="btn"
                            onclick="sendMsg('${Messages.TOGGLE_AUTOBUILD}')"
                            ${project.autoBuildEnabled ? "checked" : ""}
                            ${project.state.isEnabled ? " " : " disabled"}
                        />
                    </td>
                </tr>
                ${emptyRow}
                ${buildRow("Application Status", project.state.appState)}
                ${buildRow("Build Status", getNonNull(project.state.getBuildString(), "Not available"))}
                ${emptyRow}
                ${buildRow("Last Image Build", formatDate(project.lastImgBuild, "Not available"))}
                ${buildRow("Last Build", formatDate(project.lastBuild, "Not available"))}
                ${emptyRow}
                ${project.hasContextRoot ? buildRow("Context Root", "/" + project.contextRoot) : ""}
                ${buildRow("Application URL", getNonNull(project.appBaseUrl, "Not running"), (project.appBaseUrl != null ? Openable.WEB : undefined))}
                ${buildRow("Application Port", getNonNull(project.appPort, "Not running"))}
                ${buildRow("Debug Port", getNonNull(project.debugPort, "Not debugging"))}
                ${buildRow("Debug URL", getNonNull(project.debugUrl, "Not debugging"))}
            </table>

            <div id="bottom-section">
                <input id="delete-btn"  type="button" onclick="sendMsg('${Messages.DELETE}')" class="btn" value="Delete project"/>
                <input id="enablement-btn" type="button" onclick="sendMsg('${Messages.TOGGLE_ENABLEMENT}')" class="btn"
                    value="${(project.state.isEnabled ? "Disable" : "Enable") + " project"}"/>
            </div>
        </div>

        <script type="text/javascript">
            const vscode = acquireVsCodeApi();

            function vscOpen(element, type) {
                sendMsg("${Messages.OPEN}", { type: type, value: element.textContent });
            }

            function sendMsg(type, data = undefined) {
                vscode.postMessage({ type: type, data: data });
            }
        </script>

        </body>
        </html>
    `;
}
exports.generateHtml = generateHtml;
function getStylesheetPath() {
    return resourceScheme + Resources_1.default.getCss("project-overview.css");
}
function getMCIconPath() {
    const mcIconPath = resourceScheme + Resources_1.default.getIconPaths(Resources_1.default.Icons.Microclimate).dark;
    // Logger.log("MCIP", mcIconPath);
    return mcIconPath;
}
function buildRow(label, data, openable) {
    let td;
    if (openable != null) {
        td = `
            <td>
                <a onclick="vscOpen(this, '${openable}')">${data}</a>
            </td>
            `;
    }
    else {
        td = `<td>${data}</td>`;
    }
    // console.log("The td is ", td);
    return `
        <tr class="info-row">
            <td class="info-label">${label}:</td>
            ${td}
        </tr>
    `;
}
function getNonNull(item, fallback, maxLength) {
    let result;
    if (item == null || item === "") {
        result = fallback;
    }
    else if (item instanceof vscode.Uri && item.scheme.includes("file")) {
        result = item.fsPath;
    }
    else {
        result = item.toString();
    }
    if (maxLength != null) {
        result = result.substring(0, maxLength);
    }
    return result;
}
function formatDate(d, fallback) {
    if (MCUtil.isGoodDate(d)) {
        let dateStr = d.toLocaleDateString();
        if (dateStr === (new Date()).toLocaleDateString()) {
            dateStr = "Today";
        }
        return `${dateStr} at ${d.toLocaleTimeString()}`;
    }
    else {
        return fallback;
    }
}
//# sourceMappingURL=ProjectOverview.js.map