"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const ProjectState_1 = require("./ProjectState");
const Logger_1 = require("../../Logger");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const translator_1 = require("../../constants/strings/translator");
const StartModes_1 = require("../../constants/StartModes");
const AttachDebuggerCmd_1 = require("../../command/AttachDebuggerCmd");
const Resources_1 = require("../../constants/Resources");
const STRING_NS = StringNamespaces_1.default.PROJECT;
// States that a project cycles through during a Run mode restart
const RESTART_STATES_RUN = [
    ProjectState_1.default.AppStates.STOPPED,
    ProjectState_1.default.AppStates.STARTING,
    ProjectState_1.default.AppStates.STARTED
];
const RESTART_STATES_DEBUG = [
    ProjectState_1.default.AppStates.STOPPED,
    ProjectState_1.default.AppStates.DEBUG_STARTING,
    ProjectState_1.default.AppStates.DEBUGGING
];
/**
 * Wraps a promise which resolves (see `fulfill` function) when the project restarts, fails to restart, or has its restart time out.
 * Displays a status bar message tied to the restart promise.
 * Each project should have either one or zero of these at a time. Concurrent restarts on one project does not make sense and should be prevented.
 */
class ProjectPendingRestart {
    constructor(project, startMode, timeoutMs) {
        this.project = project;
        this.startMode = startMode;
        // Index in expectedStates pointing to the state we expect next.
        this.nextStateIndex = 0;
        Logger_1.default.d(`${project.name}: New pendingRestart into ${startMode} mode`);
        this.expectedStates = StartModes_1.default.isDebugMode(startMode) ? RESTART_STATES_DEBUG : RESTART_STATES_RUN;
        // Resolved when the restart completes or times out. Displayed in the status bar.
        const restartPromise = new Promise((resolve_) => {
            this.resolve = resolve_;
        });
        this.restartEventPromise = new Promise((resolve_) => {
            this.resolveRestartEvent = resolve_;
        });
        // Fails the restart when the timeout expires
        this.timeoutID = setTimeout(() => {
            const failReason = translator_1.default.t(STRING_NS, "restartFailedReasonTimeout", { timeoutS: timeoutMs / 1000 });
            Logger_1.default.i("Rejecting restart: " + failReason);
            this.fulfill(false, failReason);
        }, timeoutMs);
        const restartMsg = `${Resources_1.default.getOcticon(Resources_1.default.Octicons.sync, true)} ` +
            translator_1.default.t(STRING_NS, "restartingStatusMsg", { projectName: project.name, startMode: StartModes_1.default.getUserFriendlyStartMode(startMode) });
        vscode.window.setStatusBarMessage(restartMsg, restartPromise);
    }
    /**
     * Parent project object calls this in update().
     */
    onStateChange(currentState) {
        return __awaiter(this, void 0, void 0, function* () {
            if (currentState === this.expectedStates[this.nextStateIndex]) {
                this.nextStateIndex++;
                if (this.nextStateIndex === this.expectedStates.length) {
                    Logger_1.default.d("Reached restart terminal state");
                    // Might already be resolved depending on timing
                    yield this.restartEventPromise;
                    Logger_1.default.d("Done waiting for restart event");
                    // The restart was successful
                    this.fulfill(true);
                }
                else {
                    Logger_1.default.d("Restart expecting next state: " + this.expectedStates[this.nextStateIndex]);
                }
            }
        });
    }
    /**
     * Parent Project calls this when the restartResult socket event is received.
     * Should only be called once per instance of this class (ie, per restart).
     */
    onReceiveRestartEvent(success, error) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this.project.name}: pending restart received restart event, success=${success}`);
            if (!success) {
                // The restart failed in Microclimate
                this.fulfill(success, error);
            }
            else {
                if (StartModes_1.default.isDebugMode(this.startMode)) {
                    yield this.attachDebugger();
                }
            }
            if (this.resolveRestartEvent != null) {
                Logger_1.default.d("Resolving restart event promise");
                this.resolveRestartEvent();
            }
            else {
                // will never happen
                Logger_1.default.e("Null resolveRestartEvent");
            }
        });
    }
    attachDebugger() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Attaching debugger as part of restart");
            try {
                const debuggerAttached = yield AttachDebuggerCmd_1.default(this.project, true);
                if (!debuggerAttached) {
                    const debuggerAttachFailedMsg = translator_1.default.t(STRING_NS, "restartFailedReasonDebugFailure");
                    Logger_1.default.w(debuggerAttachFailedMsg);
                    // If we're debugging init, the restart fails here because it will get stuck without the debugger attach
                    if (this.startMode === StartModes_1.default.Modes.DEBUG) {
                        this.fulfill(false, debuggerAttachFailedMsg);
                    }
                }
            }
            catch (err) {
                // attachDebuggerCmd shouldn't throw/reject, but just in case:
                Logger_1.default.w("Debugger attach failed or was cancelled by user", err);
                const errMsg = err.message || err;
                vscode.window.showErrorMessage(errMsg);
            }
        });
    }
    onConnectionDisconnect() {
        this.fulfill(false, translator_1.default.t(STRING_NS, "restartFailedReasonDisconnect"));
    }
    /**
     * Resolves this class's restart promise, removing the status bar item and rendering this pending restart "done".
     * Also calls onRestartFinish which removes the Project's reference to this instance.
     *
     * Displays a success or failure message to the user depending on the value of `success`.
     */
    fulfill(success, error) {
        Logger_1.default.d(`Fulfilling pending restart for ${this.project.name}, success=${success}${error ? ", error=" + error : ""}`);
        if (this.resolve == null || this.timeoutID == null) {
            // will never happen
            Logger_1.default.e("Cannot fulfill pending restart because of an initialization failure");
            return;
        }
        this.resolve();
        if (success) {
            const successMsg = translator_1.default.t(STRING_NS, "restartSuccess", { projectName: this.project.name, startMode: StartModes_1.default.getUserFriendlyStartMode(this.startMode) });
            Logger_1.default.i(successMsg);
            vscode.window.showInformationMessage(successMsg);
        }
        else {
            let failMsg;
            if (error != null) {
                failMsg = translator_1.default.t(STRING_NS, "restartFailureWithReason", { projectName: this.project.name, startMode: StartModes_1.default.getUserFriendlyStartMode(this.startMode), reason: error });
            }
            else {
                failMsg = translator_1.default.t(STRING_NS, "restartFailure", { projectName: this.project.name, startMode: StartModes_1.default.getUserFriendlyStartMode(this.startMode) });
            }
            Logger_1.default.w(failMsg);
            vscode.window.showErrorMessage(failMsg);
        }
        clearTimeout(this.timeoutID);
        this.project.onRestartFinish();
    }
}
exports.default = ProjectPendingRestart;
//# sourceMappingURL=ProjectPendingRestart.js.map