"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../../Logger");
const StartModes_1 = require("../../constants/StartModes");
const SocketEvents_1 = require("../connection/SocketEvents");
/**
 * Represents the project's state in Microclimate. This means app state, build state, and any status details.
 * Immutable.
 */
class ProjectState {
    constructor(projectInfoPayload, 
    // Use oldState if the projectInfoPayload is missing state information (eg. from a restart success event)
    // It will be used as fallback values if the new state is null or UNKNOWN.
    oldState) {
        if (projectInfoPayload != null) {
            if (oldState != null) {
                if (projectInfoPayload[SocketEvents_1.default.Keys.APP_STATE] == null) {
                    projectInfoPayload[SocketEvents_1.default.Keys.APP_STATE] = oldState.appState.toString();
                }
                if (projectInfoPayload[SocketEvents_1.default.Keys.BUILD_STATE] == null) {
                    projectInfoPayload[SocketEvents_1.default.Keys.BUILD_STATE] = oldState.buildState.toString();
                }
                if (!projectInfoPayload[SocketEvents_1.default.Keys.BUILD_DETAIL]) {
                    projectInfoPayload[SocketEvents_1.default.Keys.BUILD_DETAIL] = oldState.buildDetail;
                }
            }
            this.appState = ProjectState.getAppState(projectInfoPayload);
            this.buildState = ProjectState.getBuildState(projectInfoPayload);
            this.buildDetail = projectInfoPayload[SocketEvents_1.default.Keys.BUILD_DETAIL] || "";
        }
        else {
            Logger_1.default.e("ProjectState received null ProjectInfo");
            this.appState = ProjectState.AppStates.UNKNOWN;
            this.buildState = ProjectState.BuildStates.UNKNOWN;
            this.buildDetail = "";
        }
    }
    equals(other) {
        return other != null &&
            this.appState === other.appState &&
            this.buildState === other.buildState &&
            this.buildDetail === other.buildDetail;
    }
    get isEnabled() {
        return ProjectState.getEnabledStates().includes(this.appState);
    }
    get isStarted() {
        return ProjectState.getStartedStates().includes(this.appState);
    }
    get isBuilding() {
        return this.buildState === ProjectState.BuildStates.BUILDING;
    }
    toString() {
        const appState = this.appState.toString();
        if (this.isEnabled) {
            let result = `[${appState}]`; // non-nls
            const buildStr = this.getBuildString();
            if (buildStr != null && buildStr.length > 0) {
                result += ` [${buildStr}]`; // non-nls
            }
            return result;
        }
        else {
            // don't show build detail for disabled projects
            return `[${appState}]`; // non-nls
        }
    }
    getBuildString() {
        if (!this.isEnabled) {
            return undefined;
        }
        let buildStateStr = "";
        if (this.buildDetail != null && this.buildDetail.trim() !== "") {
            // a detailed status is available
            buildStateStr = `${this.buildState} - ${this.buildDetail}`; // non-nls
        }
        // Don't display the build state if it's unknown
        else if (this.buildState !== ProjectState.BuildStates.UNKNOWN) {
            buildStateStr = `${this.buildState}`; // non-nls
        }
        return buildStateStr;
    }
}
exports.ProjectState = ProjectState;
(function (ProjectState) {
    // The AppStates and BuildStates string values are all exposed to the user.
    // These _should_ be translated, but since you can't put computed values in a string enum,
    // we can cross that bridge when we come to it :)
    let AppStates;
    (function (AppStates) {
        AppStates["STARTED"] = "Running";
        AppStates["STARTING"] = "Starting";
        AppStates["STOPPING"] = "Stopping";
        AppStates["STOPPED"] = "Stopped";
        AppStates["DEBUGGING"] = "Debugging";
        AppStates["DEBUG_STARTING"] = "Starting - Debug";
        AppStates["DISABLED"] = "Disabled";
        AppStates["UNKNOWN"] = "Unknown";
    })(AppStates = ProjectState.AppStates || (ProjectState.AppStates = {}));
    let BuildStates;
    (function (BuildStates) {
        BuildStates["BUILD_SUCCESS"] = "Build Succeeded";
        BuildStates["BUILDING"] = "Building";
        BuildStates["BUILD_FAILED"] = "Build Failed";
        BuildStates["BUILD_QUEUED"] = "Build Queued";
        BuildStates["UNKNOWN"] = "Unknown";
    })(BuildStates = ProjectState.BuildStates || (ProjectState.BuildStates = {}));
    function getEnabledStates() {
        return [
            AppStates.STARTED,
            AppStates.STARTING,
            AppStates.STOPPING,
            AppStates.STOPPED,
            AppStates.DEBUGGING,
            AppStates.DEBUG_STARTING,
            AppStates.UNKNOWN
        ];
    }
    ProjectState.getEnabledStates = getEnabledStates;
    function getStartedStates() {
        return [
            ProjectState.AppStates.STARTED,
            ProjectState.AppStates.DEBUGGING
        ];
    }
    ProjectState.getStartedStates = getStartedStates;
    function getStartingStates() {
        return [
            ProjectState.AppStates.STARTING,
            ProjectState.AppStates.DEBUG_STARTING
        ];
    }
    ProjectState.getStartingStates = getStartingStates;
    function getDebuggableStates() {
        return [
            ProjectState.AppStates.DEBUGGING,
            ProjectState.AppStates.DEBUG_STARTING
        ];
    }
    ProjectState.getDebuggableStates = getDebuggableStates;
    /**
     * Convert Microclimate's project info object into a ProjectState.
     */
    function getAppState(projectInfoPayload) {
        // Logger.log("PIP", projectInfoPayload);
        const appStatus = projectInfoPayload[SocketEvents_1.default.Keys.APP_STATE] || "";
        const closedState = projectInfoPayload[SocketEvents_1.default.Keys.CLOSED_STATE];
        const startMode = projectInfoPayload[SocketEvents_1.default.Keys.START_MODE];
        // Logger.log(`Convert - appStatus=${appStatus}, closedState=${closedState}, startMode=${startMode}`);
        // First, check if the project is closed (aka Disabled)
        if (closedState === "closed" || appStatus === AppStates.DISABLED) { // non-nls
            return ProjectState.AppStates.DISABLED;
        }
        // Now, check the app states. Compare against both the value we expect from MC,
        // as well as our own possible values, in case we used the fallbackState in the constructor.
        else if (appStatus === "started" || appStatus === AppStates.DEBUGGING || appStatus === AppStates.STARTED) { // non-nls
            if (startMode != null && StartModes_1.default.isDebugMode(startMode)) {
                return ProjectState.AppStates.DEBUGGING;
            }
            return ProjectState.AppStates.STARTED;
        }
        else if (appStatus === "starting" || appStatus === AppStates.STARTING || appStatus === AppStates.DEBUG_STARTING) { // non-nls
            if (startMode != null && StartModes_1.default.isDebugMode(startMode)) {
                return ProjectState.AppStates.DEBUG_STARTING;
            }
            return ProjectState.AppStates.STARTING;
        }
        else if (appStatus === "stopping" || appStatus === AppStates.STOPPING) { // non-nls
            return ProjectState.AppStates.STOPPING;
        }
        else if (appStatus === "stopped" || appStatus === AppStates.STOPPED) { // non-nls
            return ProjectState.AppStates.STOPPED;
        }
        else if (appStatus === "unknown" || appStatus === "" || appStatus === AppStates.UNKNOWN) { // non-nls
            return ProjectState.AppStates.UNKNOWN;
        }
        else {
            Logger_1.default.e("Unknown app state:", appStatus);
            return ProjectState.AppStates.UNKNOWN;
        }
    }
    ProjectState.getAppState = getAppState;
    function getBuildState(projectInfoPayload) {
        const buildStatus = projectInfoPayload[SocketEvents_1.default.Keys.BUILD_STATE];
        if (buildStatus === "success" || buildStatus === BuildStates.BUILD_SUCCESS) { // non-nls
            return BuildStates.BUILD_SUCCESS;
        }
        else if (buildStatus === "inProgress" || buildStatus === BuildStates.BUILDING) { // non-nls
            return BuildStates.BUILDING;
        }
        else if (buildStatus === "queued" || buildStatus === BuildStates.BUILD_QUEUED) { // non-nls
            return BuildStates.BUILD_QUEUED;
        }
        else if (buildStatus === "failed" || buildStatus === BuildStates.BUILD_FAILED) { // non-nls
            return BuildStates.BUILD_FAILED;
        }
        else if (buildStatus == null || buildStatus.toLowerCase() === "unknown") { // non-nls
            return BuildStates.UNKNOWN;
        }
        else {
            Logger_1.default.e("Unknown build state:", buildStatus);
            return BuildStates.UNKNOWN;
        }
    }
    ProjectState.getBuildState = getBuildState;
})(ProjectState = exports.ProjectState || (exports.ProjectState = {}));
exports.default = ProjectState;
//# sourceMappingURL=ProjectState.js.map