"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const Resources_1 = require("../../constants/Resources");
const Logger_1 = require("../../Logger");
class ProjectType {
    constructor(internalType, language) {
        this.internalType = internalType;
        this.language = language;
        this.type = ProjectType.getType(internalType, language);
        // this.userFriendlyType = ProjectType.getUserFriendlyType(this.type);
        this.debugType = ProjectType.getDebugType(this.type);
        this.icon = ProjectType.getProjectIcon(this.type);
    }
    toString() {
        return this.type.toString();
    }
    /**
     *
     * @param interalType A Microclimate internal project type.
     */
    static getType(interalType, language) {
        if (interalType === this.InternalTypes.MICROPROFILE) {
            return ProjectType.Types.MICROPROFILE;
        }
        else if (interalType === this.InternalTypes.SPRING) {
            return ProjectType.Types.SPRING;
        }
        else if (interalType === this.InternalTypes.NODE) {
            return ProjectType.Types.NODE;
        }
        else if (interalType === this.InternalTypes.SWIFT) {
            return ProjectType.Types.SWIFT;
        }
        else if (interalType === this.InternalTypes.DOCKER) {
            if (language === this.Languages.PYTHON) {
                return ProjectType.Types.PYTHON;
            }
            else if (language === this.Languages.GO) {
                return ProjectType.Types.GO;
            }
            else {
                return ProjectType.Types.GENERIC_DOCKER;
            }
        }
        else {
            Logger_1.default.e(`Unrecognized project type ${interalType}`);
            return ProjectType.Types.UNKNOWN;
        }
    }
    /**
     * Get the corresponding VSCode debug configuration "type" value.
     * Returns undefined if we don't know how to debug this project type.
     */
    static getDebugType(type) {
        switch (type) {
            case ProjectType.Types.MICROPROFILE:
            case ProjectType.Types.SPRING:
                return this.DebugTypes.JAVA;
            case ProjectType.Types.NODE:
                return this.DebugTypes.NODE;
            default:
                return undefined;
        }
    }
    static getProjectIcon(type) {
        switch (type) {
            case ProjectType.Types.MICROPROFILE:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Microprofile);
            case ProjectType.Types.SPRING:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Spring);
            case ProjectType.Types.NODE:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Node);
            case ProjectType.Types.SWIFT:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Swift);
            case ProjectType.Types.PYTHON:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Python);
            case ProjectType.Types.GO:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Go);
            case ProjectType.Types.GENERIC_DOCKER:
                // Could return the Java icon for Lagom
                // This is our fall-back, we could possibly use a more generic icon.
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Docker);
            default:
                return Resources_1.default.getIconPaths(Resources_1.default.Icons.Generic);
        }
    }
    /*
    private static getUserFriendlyType(type: ProjectType.Types): string {
        // For docker projects, return the language
        /*
        if (type === ProjectType.Types.GENERIC_DOCKER && language != null) {
            return uppercaseFirstChar(language);
        }

        // For all other types, the enum's string value is user-friendly
        return type.toString();
    }*/
    get providesBuildLog() {
        return !ProjectType.PROJECTS_WITHOUT_BUILDLOGS.includes(this.type);
    }
}
exports.ProjectType = ProjectType;
(function (ProjectType) {
    // These are the project types as exposed to the user.
    // String value must be user-friendly!
    let Types;
    (function (Types) {
        Types["MICROPROFILE"] = "Microprofile";
        Types["SPRING"] = "Spring";
        Types["NODE"] = "Node.js";
        Types["SWIFT"] = "Swift";
        Types["PYTHON"] = "Python";
        Types["GO"] = "Go";
        Types["GENERIC_DOCKER"] = "Docker";
        Types["UNKNOWN"] = "Unknown";
    })(Types = ProjectType.Types || (ProjectType.Types = {}));
    // non-nls-section-start
    // possible values of the "projectType" or "buildType" internal attribute
    let InternalTypes;
    (function (InternalTypes) {
        InternalTypes["MICROPROFILE"] = "liberty";
        InternalTypes["SPRING"] = "spring";
        InternalTypes["NODE"] = "nodejs";
        InternalTypes["SWIFT"] = "swift";
        InternalTypes["DOCKER"] = "docker";
    })(InternalTypes = ProjectType.InternalTypes || (ProjectType.InternalTypes = {}));
    // Possible values of the "language" internal attribute
    // could be others, but if they're not in this list we'll just treat them as generic docker
    let Languages;
    (function (Languages) {
        Languages["JAVA"] = "java";
        Languages["NODE"] = "nodejs";
        Languages["SWIFT"] = "swift";
        Languages["PYTHON"] = "python";
        Languages["GO"] = "go";
    })(Languages = ProjectType.Languages || (ProjectType.Languages = {}));
    // VSCode debug types, used as the "type" attribute in a debug launch.
    let DebugTypes;
    (function (DebugTypes) {
        DebugTypes["JAVA"] = "java";
        DebugTypes["NODE"] = "node";
    })(DebugTypes = ProjectType.DebugTypes || (ProjectType.DebugTypes = {}));
    // non-nls-section-end
    ProjectType.PROJECTS_WITHOUT_BUILDLOGS = [
        ProjectType.Types.NODE
    ];
})(ProjectType = exports.ProjectType || (exports.ProjectType = {}));
exports.default = ProjectType;
//# sourceMappingURL=ProjectType.js.map