"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const StartModes_1 = require("../../constants/StartModes");
const Endpoints_1 = require("../../constants/Endpoints");
const Logger_1 = require("../../Logger");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const translator_1 = require("../../constants/strings/translator");
const MCUtil = require("../../MCUtil");
const STRING_NS = StringNamespaces_1.default.REQUESTS;
var Requester;
(function (Requester) {
    function requestProjectRestart(project, startMode) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                startMode: startMode.toString()
            };
            const url = Endpoints_1.default.getProjectEndpoint(project.connection, project.id, Endpoints_1.default.RESTART_ACTION);
            const restartMsg = translator_1.default.t(STRING_NS, "restartIntoMode", { startMode: StartModes_1.default.getUserFriendlyStartMode(startMode) });
            return doProjectRequest(project, url, body, request.post, restartMsg);
        });
    }
    Requester.requestProjectRestart = requestProjectRestart;
    function requestBuild(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                action: "build" // non-nls
            };
            const url = Endpoints_1.default.getProjectEndpoint(project.connection, project.id, Endpoints_1.default.BUILD_ACTION);
            // return doProjectRequest(project, url, body, request.post, "Build");
            const buildMsg = translator_1.default.t(STRING_NS, "build");
            return doProjectRequest(project, url, body, request.post, buildMsg)
                // This is a workaround for the Build action not refreshing validation state.
                // Will be fixed by https://github.ibm.com/dev-ex/iterative-dev/issues/530
                .then(() => requestValidate(project, true));
        });
    }
    Requester.requestBuild = requestBuild;
    function requestToggleAutoBuild(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newAutoBuild = !project.autoBuildEnabled;
            // user-friendly action
            const autoBuildMsgKey = newAutoBuild ? "autoBuildEnable" : "autoBuildDisable"; // non-nls
            const newAutoBuildUserStr = translator_1.default.t(STRING_NS, autoBuildMsgKey);
            // action we'll put into the request body   https://github.ibm.com/dev-ex/portal/wiki/API:-Build
            const newAutoBuildAction = newAutoBuild ? "enableautobuild" : "disableautobuild"; // non-nls
            const body = {
                action: newAutoBuildAction
            };
            const url = Endpoints_1.default.getProjectEndpoint(project.connection, project.id, Endpoints_1.default.BUILD_ACTION);
            return doProjectRequest(project, url, body, request.post, newAutoBuildUserStr)
                .then((result) => {
                if (result != null && MCUtil.isGoodStatusCode(result.statusCode)) {
                    Logger_1.default.d("Received good status from autoBuild request, new auto build is: " + newAutoBuild);
                    project.setAutoBuild(newAutoBuild);
                }
            });
        });
    }
    Requester.requestToggleAutoBuild = requestToggleAutoBuild;
    function requestToggleEnablement(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newEnablement = !project.state.isEnabled;
            const newEnablementMsgKey = newEnablement ? "projectEnable" : "projectDisable"; // non-nls
            const newEnablementStr = translator_1.default.t(STRING_NS, newEnablementMsgKey);
            const url = Endpoints_1.default.getProjectEndpoint(project.connection, project.id, Endpoints_1.default.getEnablementAction(newEnablement));
            return doProjectRequest(project, url, {}, request.put, newEnablementStr);
        });
    }
    Requester.requestToggleEnablement = requestToggleEnablement;
    function requestValidate(project, silent) {
        return __awaiter(this, void 0, void 0, function* () {
            const [url, body] = assembleValidateRequest(project, false);
            const userOperation = silent ? undefined : translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "validate");
            return doProjectRequest(project, url, body, request.post, userOperation);
        });
    }
    Requester.requestValidate = requestValidate;
    function requestGenerate(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const [url, body] = assembleValidateRequest(project, true);
            const generateMsg = translator_1.default.t(STRING_NS, "generateMissingFiles");
            return doProjectRequest(project, url, body, request.post, generateMsg)
                // request a validate after the generate so that the validation errors go away faster
                .then(() => requestValidate(project, true));
        });
    }
    Requester.requestGenerate = requestGenerate;
    // In 1901, the validation API changed
    const validationAPIChangeVersion = 1901;
    /**
     * Get the URL and request body for either a Validate or Generate request, they are very similar.
     */
    function assembleValidateRequest(project, generate) {
        let url;
        const body = {
            projectType: project.type.internalType,
        };
        if (generate) {
            body.autoGenerate = true;
        }
        if (project.connection.version >= validationAPIChangeVersion) {
            url = Endpoints_1.default.getProjectEndpoint(project.connection, project.id, generate ? Endpoints_1.default.GENERATE : Endpoints_1.default.VALIDATE);
        }
        else {
            url = Endpoints_1.default.getEndpoint(project.connection, generate ? Endpoints_1.default.GENERATE_OLD : Endpoints_1.default.VALIDATE_OLD);
            body.projectID = project.id;
        }
        return [url, body];
    }
    function requestDelete(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = Endpoints_1.default.getProjectEndpoint(project.connection, project.id, "");
            const deleteMsg = translator_1.default.t(STRING_NS, "delete");
            return doProjectRequest(project, url, {}, request.delete, deleteMsg);
        });
    }
    Requester.requestDelete = requestDelete;
    /**
     * Perform a REST request of the type specific by `requestFunc` to the project endpoint for the given project.
     * Displays a message to the user that the request succeeded if userOperationName is not null.
     * Always displays a message to the user in the case of an error.
     */
    function doProjectRequest(project, url, body, requestFunc, userOperationName) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`Doing ${userOperationName != null ? userOperationName + " " : ""}request to ${url}`);
            const options = {
                json: true,
                body: body,
                resolveWithFullResponse: true
            };
            return requestFunc(url, options)
                .then((result) => {
                Logger_1.default.d(`Response code ${result.statusCode} from ${userOperationName} ${requestFunc.name.toUpperCase()} request for ${project.name}`);
                if (userOperationName != null) {
                    vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "requestSuccess", { operationName: userOperationName, projectName: project.name }));
                }
                return result;
            })
                .catch((err) => {
                Logger_1.default.w(`Error doing ${userOperationName} project request for ${project.name}:`, err);
                // If the server provided a specific message, present the user with that,
                // otherwise show them the whole error (but it will be ugly)
                // err.error.msg, then err.error, then the whole err
                const errMsg = err.error ? (err.error.msg ? err.error.msg : err.error) : JSON.stringify(err);
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "requestFail", { operationName: userOperationName, projectName: project.name, err: errMsg }));
                return err;
            });
        });
    }
})(Requester || (Requester = {}));
exports.default = Requester;
//# sourceMappingURL=Requester.js.map