"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Logger_1 = require("../../Logger");
const Project_1 = require("./Project");
const Requester_1 = require("./Requester");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const ProjectType_1 = require("./ProjectType");
const Commands_1 = require("../../constants/Commands");
var Validator;
(function (Validator) {
    function validate(project, validationResult) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`Validating ${project.name}`);
            if (validationResult != null && validationResult.length > 0) {
                Logger_1.default.i(`ValidationResult is:`, validationResult);
            }
            // clicking on the error will take you to this URI
            // it's the project folder path -
            // unfortunately vscode gives an error that it can't be opened when clicked, so this can likely be improved
            const diagnosticUri = project.localPath;
            const oldDiagnostics = Project_1.default.diagnostics.get(diagnosticUri) || [];
            const newDiagnostics = [];
            // For each validation problem, see if we already have an error for it. If so, do nothing.
            // If we don't, create an error and display a pop-up notifying the user of the new error.
            for (const validationProblem of validationResult) {
                // Log.i("ValidationProblem:", validationProblem);
                const diagnosticMsg = validationProblem.details;
                const existingDiagnostic = oldDiagnostics.find((d) => d.message === diagnosticMsg);
                if (existingDiagnostic != null) {
                    // we already have a marker for this error, we can re-use it and continue to the next one
                    // and don't need to display the error pop-up again
                    newDiagnostics.push(existingDiagnostic);
                    continue;
                }
                const sev = validationProblem.severity === "error" ? vscode.DiagnosticSeverity.Error : vscode.DiagnosticSeverity.Warning; // non-nls
                const diagnostic = new vscode.Diagnostic(new vscode.Range(0, 0, 0, 0), diagnosticMsg, sev);
                diagnostic.source = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "microclimateName");
                newDiagnostics.push(diagnostic);
                const seeProblemsViewMsg = translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "seeProblemsView");
                const filePath = validationProblem.filepath;
                const popupErrMsg = `${diagnostic.source}: ${validationProblem.label} ${filePath}. ${seeProblemsViewMsg}`; // non-nls
                const problemsBtn = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "focusOnProblems");
                // Allow the user to generate missing files.
                // Generate only works for dockerfile, so only display the Generate button if that's what's missing.
                // Full list of supported files: https://www.npmjs.com/package/generator-ibm-cloud-enablement#artifacts
                if (validationProblem.filename === "Dockerfile" && project.type.internalType !== ProjectType_1.default.InternalTypes.DOCKER) { // non-nls
                    const generateBtn = translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "generateFilesBtn");
                    vscode.window.showErrorMessage(popupErrMsg, generateBtn, problemsBtn)
                        .then((response) => {
                        if (response === generateBtn) {
                            Requester_1.default.requestGenerate(project);
                        }
                        // If the user clicks this, they miss their chance to Generate, which might be frustrating.
                        // Might not want to show this if we can Generate.
                        else if (response === problemsBtn) {
                            vscode.commands.executeCommand(Commands_1.default.VSC_FOCUS_PROBLEMS);
                        }
                    });
                }
                else {
                    // show the validation error without the Generate button.
                    vscode.window.showErrorMessage(popupErrMsg, problemsBtn)
                        .then((response) => {
                        if (response === problemsBtn) {
                            vscode.commands.executeCommand(Commands_1.default.VSC_FOCUS_PROBLEMS);
                        }
                    });
                }
            }
            Project_1.default.diagnostics.set(diagnosticUri, newDiagnostics);
        });
    }
    Validator.validate = validate;
})(Validator || (Validator = {}));
exports.default = Validator;
//# sourceMappingURL=Validator.js.map